indexing
	description: 
	
		"Message to send result of a round to clients"
		
	date: "$Date: 2006-03-01 15:30:10 +0100 (Wed, 01 Mar 2006) $"
	revision: "$Revision: 1914 $"

class BB_MP_GAME_RESULT

inherit

	EM_NET_2PC_EVENT_OBJECT
	
		redefine
			make_set_type
		end
	
create {BB_MP_OBJECT_TYPES}

	make_set_type
		
feature {NONE} -- Initialization

	make_set_type (a_type_id: INTEGER) is
			-- Set `create_type_id' to `a_type_id'.
		do
			precursor (a_type_id)
			
			create game_result.make
			
		end

feature -- Status report

	game_result: DS_LINKED_LIST [BB_MP_PLAYER_INFO]
			-- game_result list
			
	most_remaining_blocks: INTEGER
			-- number of blocks of player with most remaining blocks
			
feature -- Status setting

	set_game_result (a_game_result: DS_LINKED_LIST [BB_MP_PLAYER_INFO]) is
			-- set `game_result' to `a_game_result'
		require
			a_game_result /= Void
		do
			game_result := a_game_result
		ensure
			value_set: game_result.is_equal (a_game_result)
		end
		
	set_most_remaining_blocks (an_integer: INTEGER) is
			-- set `most_remaining_blocks'
		do
			most_remaining_blocks := an_integer
		ensure
			value_set: most_remaining_blocks = an_integer
		end
		
feature -- Serialization

	serialize (a_serializer: EM_NET_SERIALIZER) is
			-- Serialize to the given stream using `a_serializer'
		do
			a_serializer.put_integer (id)
			a_serializer.put_integer (most_remaining_blocks)
			a_serializer.put_integer (game_result.count)
			from			
				game_result.start
			until
				game_result.after
			loop
				a_serializer.put_integer (game_result.item_for_iteration.score)
				a_serializer.put_line (game_result.item_for_iteration.player_name)
				a_serializer.put_integer (game_result.item_for_iteration.remaining_blocks)
				a_serializer.put_integer (game_result.item_for_iteration.number_of_wins)
				a_serializer.put_integer (game_result.item_for_iteration.lifes)
				game_result.forth
			end
		end
		
	serialization_byte_count: INTEGER is
			-- Count of bytes needed for a successful serialization
		do
			Result := 12
			
			from	
				game_result.start
			until
				game_result.after
			loop
				Result := 16 + game_result.item_for_iteration.player_name.count + 1
				
				game_result.forth
			end
		end
		
	unserialize (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Unserialize from a given stream using `an_unserializer'
		local
			i, number_of_players: INTEGER
			temp_score: INTEGER
		do
			an_unserializer.read_integer
			id := an_unserializer.last_integer
			
			an_unserializer.read_integer
			most_remaining_blocks := an_unserializer.last_integer
			
			an_unserializer.read_integer
			number_of_players := an_unserializer.last_integer
			
			from
				i := 0
			until
				i >= number_of_players
			loop
				an_unserializer.read_integer
				temp_score := an_unserializer.last_integer
				
				an_unserializer.read_line
				game_result.force_last (create {BB_MP_PLAYER_INFO}.make (an_unserializer.last_string,"",temp_score))
				
				an_unserializer.read_integer
				game_result.last.set_remaining_blocks (an_unserializer.last_integer)
				
				an_unserializer.read_integer
				game_result.last.set_number_of_wins (an_unserializer.last_integer)
				
				an_unserializer.read_integer
				game_result.last.set_lifes (an_unserializer.last_integer)
				
				i := i + 1
			end
			
		end
		
end

