indexing
	description: 
	
		"Message containing current ranking"
		
	date: "$Date: 2006-03-01 15:30:10 +0100 (Wed, 01 Mar 2006) $"
	revision: "$Revision: 1914 $"

class BB_MP_RANKING

inherit
	
	EM_NET_2PC_EVENT_OBJECT
		redefine
			make_set_type
		end
	
create {BB_MP_OBJECT_TYPES}

	make_set_type
		
feature {NONE} -- Initialization

	make_set_type (a_type_id: INTEGER) is
			-- Set `create_type_id' to `a_type_id'.
		do
			precursor (a_type_id)
			
			create ranking.make
		end

feature -- Status report

	ranking: DS_LINKED_LIST [BB_MP_PLAYER_INFO]
			-- ranking list
			
feature -- Status setting

	set_ranking (a_ranking: DS_LINKED_LIST [BB_MP_PLAYER_INFO]) is
			-- set `ranking' to `a_ranking'
		require
			a_ranking /= Void
		do
			ranking := a_ranking
		ensure
			value_set: ranking.is_equal (a_ranking)
		end
		
feature -- Serialization

	serialize (a_serializer: EM_NET_SERIALIZER) is
			-- Serialize to the given stream using `a_serializer'
		do
			a_serializer.put_integer (id)
			a_serializer.put_integer (ranking.count)
			from			
				ranking.start
			until
				ranking.after
			loop
				a_serializer.put_integer (ranking.item_for_iteration.score)
				a_serializer.put_line (ranking.item_for_iteration.player_name)
				a_serializer.put_integer (ranking.item_for_iteration.remaining_blocks)
				a_serializer.put_integer (ranking.item_for_iteration.number_of_wins)
				a_serializer.put_integer (ranking.item_for_iteration.lifes)
				ranking.forth
			end
		end
		
	serialization_byte_count: INTEGER is
			-- Count of bytes needed for a successful serialization
		do
			Result := 8
			
			from	
				ranking.start
			until
				ranking.after
			loop
				Result := 16 + ranking.item_for_iteration.player_name.count + 1
				
				ranking.forth
			end
		end
		
	unserialize (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Unserialize from a given stream using `an_unserializer'
		local
			i, number_of_players: INTEGER
			temp_score: INTEGER
		do
			an_unserializer.read_integer
			id := an_unserializer.last_integer
			
			an_unserializer.read_integer
			number_of_players := an_unserializer.last_integer
			
			from
				i := 0
			until
				i >= number_of_players
			loop
				an_unserializer.read_integer
				temp_score := an_unserializer.last_integer
				
				an_unserializer.read_line
				ranking.force_last (create {BB_MP_PLAYER_INFO}.make (an_unserializer.last_string,"",temp_score))
				
				an_unserializer.read_integer
				ranking.last.set_remaining_blocks (an_unserializer.last_integer)
				
				an_unserializer.read_integer
				ranking.last.set_number_of_wins (an_unserializer.last_integer)
				
				an_unserializer.read_integer
				ranking.last.set_lifes (an_unserializer.last_integer)
				
				i := i + 1
			end
			
		end
		
end
