indexing
	description: "A Simple Interface for the elfractal application"
	author: "Lars Krapf"
	date: "$Date: 2007-03-13 23:06:07 +0100 (Tue, 13 Mar 2007) $"
	revision: "$Revision: 2554 $"

class
	EF_SIMPLE_PANEL

inherit
	EM_PANEL
	EM_SHARED_THEME
		export {NONE} all end
	EM_SHARED_BITMAP_FACTORY

create
	make

feature -- Initialization

	make(a_display: EF_3D_DISPLAY) is
		require
			a_display: a_display /= Void
		do
			panel_view := a_display
			make_from_dimension(200,530)
			set_background(create {EM_BITMAP_BACKGROUND}.make_from_file("menu_panel.png"))
			set_foreground_color(create {EM_COLOR}.make_with_rgb(200,200,0))

			-- Examples
			create label_examples.make_from_text("Examples:")
			label_examples.set_foreground_color (foreground_color)
			label_examples.set_position(10,60)
			add_widget(label_examples)

			create list_examples.make_empty
			list_examples.set_dimension(180,20)
			list_examples.set_position(10,80)
			list_examples.put ("Koch")
			list_examples.put ("Rainbow")
			list_examples.put ("Organic")
			list_examples.put ("Multikoch")
			list_examples.put ("Monster")
			list_examples.selection_changed_event.subscribe (agent change_example)
			add_widget(list_examples)

			-- Help Text
			create label_help.make_from_text (text_help)
			label_help.set_multilined(True)
			label_help.set_dimension(182,150)
			label_help.set_foreground_color(create {EM_COLOR}.make_with_rgb(160,160,0))
			label_help.set_border (create {EM_LINE_BORDER}.make (foreground_color, 1))
			label_help.set_position(9,110)
			add_widget(label_help)

			-- # of Iterations
			create slider_iterations.make_from_range_horizontal (0,10)
			slider_iterations.set_position(80,280)
			slider_iterations.set_dimension (180,20)
			slider_iterations.set_width (110)
			slider_iterations.set_current_value (4)
			slider_iterations.set_transparent
			add_widget(slider_iterations)

			create label_iterations.make_from_text("Iterations:")
			label_iterations.set_position(10,280)
			label_iterations.set_foreground_color (foreground_color)
			add_widget(label_iterations)

			-- angle
			create slider_angle.make_from_range_horizontal (0, 360)
			slider_angle.set_dimension (180,20)
			slider_angle.set_position(80,310)
			slider_angle.set_width (110)
			slider_angle.set_current_value (4)
			slider_angle.set_transparent
			add_widget(slider_angle)

			create label_angle.make_from_text("Angle:")
			label_angle.set_position(10,310)
			label_angle.set_foreground_color (foreground_color)
			add_widget(label_angle)

			-- Input Field for System
			create label_f.make_from_text ("F:=")
			label_f.set_position(10,340)
			label_f.set_foreground_color (foreground_color)
			add_widget(label_f)

			create textbox_system.make_from_size(19)
			textbox_system.set_position(33,340)
			add_widget(textbox_system)

			-- Compile Button
			create button_compile.make_from_text("compile")
			button_compile.set_position(133,373)
			button_compile.mouse_clicked_event.subscribe(agent compile_event)
			add_widget(button_compile)

			-- Animate Angle
			create checkbox_animate.make_from_text("Animate")
			checkbox_animate.set_foreground_color (foreground_color)
			checkbox_animate.set_position(30,373)
			checkbox_animate.checked_event.subscribe (agent panel_view.set_animated(True))
			checkbox_animate.unchecked_event.subscribe (agent panel_view.set_animated(False))
			add_widget(checkbox_animate)

			-- Event Image
			Bitmap_Factory.create_bitmap_from_image ("event_image.jpg")
			Bitmap_Factory.last_bitmap.stretch (180/Bitmap_Factory.last_bitmap.width, 180/Bitmap_Factory.last_bitmap.width)
			create button_event.make_from_image (Bitmap_Factory.last_bitmap)

			button_event.set_position(8,522-button_event.height)
			add_widget(button_event)
		end

feature -- Implementation
	compile_event(an_event: EM_MOUSEBUTTON_EVENT) is
			-- Compile Button has been clicked
		do
			compile
		end

	compile is
			-- Compile the Script and redraw
		do

			panel_view.set_system_string(create {STRING}.make_from_string(textbox_system.text))
			panel_view.set_iterations(slider_iterations.current_value)
			panel_view.set_angle(slider_angle.current_value)
			panel_view.create_script
		end

	change_example(example_string: STRING) is
			-- Change the Example and redraw
		require
			example_string: example_string /= Void
		do
			if example_string.is_equal ("Koch") then
				textbox_system.set_text("FLFRRFLF")
				slider_iterations.set_current_value(6)
				slider_angle.set_current_value(60)
				compile
			elseif example_string.is_equal ("Rainbow") then
				textbox_system.set_text("F[LF][RF][UF][DF]F")
				slider_iterations.set_current_value(5)
				slider_angle.set_current_value(47)
			elseif example_string.is_equal ("Organic") then
				textbox_system.set_text("[LF]DF[RF]F[DDF]FR[UF]")
				slider_iterations.set_current_value(5)
				slider_angle.set_current_value(64)
			elseif example_string.is_equal ("Multikoch") then
				textbox_system.set_text("[LK][RK][UK][DK]")
				panel_view.set_user_commands_string ("K=KLKRKRKLK")
				slider_iterations.set_current_value (6)
				slider_angle.set_current_value(68)
			elseif example_string.is_equal ("Monster") then
				textbox_system.set_text("F=Kopf[RBein1][LBein1][UBein2][DBein2]")
				panel_view.set_user_commands_string ("Kopf=Blue+Kopf[Blue+LKopfULKopf][Blue-RKopfDRKopf]%NBein1=Red+Bein1LLBein1[RRBein2]%NBein2=Green+Bein2LUBein2[DRBein2][DLBein1]")
				slider_iterations.set_current_value(4)
				slider_angle.set_current_value(110)
			end
			compile
		end

feature -- Widgets

	-- the help text to display
	text_help: STRING is " Builtin Commands:%N %N   F: Step Forward%N   L: Turn left%N   R: Turn right%N   U: Turn up%N   D: Turn down%N   [: Save state%N   ]: Restore state%N"

	-- text_list
	list_examples: EM_COMBOBOX [STRING]

	-- the 3d panel
	panel_view: EF_3D_DISPLAY

	-- labels
	label_angle,label_examples,label_help,label_iterations,label_f: EM_LABEL

	-- textboxes
	textbox_system: EM_TEXTBOX

	-- sliders
	slider_iterations: EM_SLIDER
	slider_angle: EM_SLIDER

	-- buttons
	button_compile: EM_BUTTON

	-- checkbox
	checkbox_animate: EM_CHECKBOX

	-- Event Image
	button_event: EM_BUTTON

end
