indexing
	description: "load a game"
	author: "Markus Neidhart and UC Team"
	date: "$Date: 2007-04-13 15:52:10 +0200 (Fri, 13 Apr 2007) $"
	revision: "$Revision: 2562 $"

class
	LOAD_GAME

inherit
	MENU
		redefine
			initialize_scene
		end

feature -- Initialize

	load: UC_LOAD_PUZZLE

	initialize_scene is
			-- Initialize the scene
		local
			bitmap: EM_BITMAP
			title: EM_STRING
		do
			Precursor
			-- Build the menu entries
			create menu_entries.make (1, 1)

			menu_entries.put (create {EM_STRING}.make ("Back", normal_font), 1)
			menu_entries.item (1).set_x_y (512 - menu_entries.item (1).width // 2, 650)

			-- Build the background
			bitmap_factory.create_empty_bitmap (width, height)
				check
					todo_proper_error_handling: bitmap_factory.last_bitmap /= Void
				end
			bitmap := bitmap_factory.last_bitmap
			bitmap.fill (uc_background_color)
			bitmap.set_x_y (1, 1)
			scene.extend (bitmap)

			selected_nr := 1
			menu_entries.item (selected_nr).set_font (select_font)
			add_menu_entries_to_scene

			-- Build the title
			create title.make ("Load your puzzle!", normal_font)
			scene.extend (title)
			title.set_x_y ((width - title.width) // 2, 10)

			-- Build the text field
			create text_field.make (event_loop)
			--text_field.start_cursor_animation
			scene.extend (text_field)
			text_field.set_x_y (150, 250)

			-- Build the `load'-Button
			create load_button.make_with_name ("Load Puzzle", event_loop)
			scene.extend (load_button)
			load_button.set_x_y (500, 252)

			-- build the error message
			create error_string.make ("", uc_gui_elements_default_font)
			scene.extend(error_string)
			error_string.set_x_y (150, 200)

			-- subscribe the event for loading the puzzle
			text_field.return_key_pressed_event.subscribe (agent load_puzzle)
			load_button.button_clicked_event.subscribe (agent load_button_clicked)

		end

	set_situation (sit: PUZZLE_START_WINDOW) is
			-- set situation to `sit'
		require
			sit_not_void: sit /= Void
		do
			situation := sit
		ensure
			situation_set: situation = sit
		end

feature {NONE} -- Implementation

	situation: PUZZLE_START_WINDOW -- actual situation, will be set in UC_INTERFACE, which have it as last_scene
	text_field: TEXT_FIELD
	load_button: BUTTON
	psw: PUZZLE_START_WINDOW

feature {NONE} -- Event handler

	load_puzzle (val: STRING) is
			-- load the puzzle
		require
			val_not_Void: val /= Void
		local
			error_code : INTEGER
		do
			error_code := input_check
			if error_code = 0 then
				create load.make (val)
				psw := load.read_file
				psw.set_load
				event_loop.stop
				on_select
			else
		   	 	-- input was somehow wrong, reset and display information
		   	 	error_message(error_code)
			end
		end

	load_button_clicked is
			-- click on load button
		do
			load_puzzle (text_field.value)
		end



	on_select is
			-- Set the `next_scene' according to `selected_nr' here
		local
			menu: MENU
		do
			if selected_nr = 1 then -- Back
				menu ?= last_scene
				menu.set_next_scene (Void)
				menu.set_last_scene (psw)
				menu.restart_event_loop
				menu.empty_main_container
				next_scene := menu
			end
		end

feature -- Input checking

	input_check : INTEGER is
			-- checks given input and returns error_code
			local
				file: PLAIN_TEXT_FILE
				file2: PLAIN_TEXT_FILE
			do
				if text_field.value.is_empty then
					result := 1
				else
					create file.make (text_field.value)
					create file2.make (text_field.value + ".puz")
					if (not file.exists) and then (not file2.exists) then
						result := 2
					else
						result := 0
					end
				end
			end

	error_message(error_code: INTEGER) is
			-- put error message to given error_code on screen
			local
				a_message : STRING -- this is the error message
			do
				inspect error_code
				when 1 then
					a_message := "Please fill in the textfield"
				when 2 then
					a_message := "File not found"
				else
					a_message := ""
				end
				error_string.set_value (a_message)
			end

	error_string : EM_STRING
			-- this is the error message


end -- class LOAD_GAME
