indexing
	description: "This contains all puzzle_parts of a puzzle."
	author: "Stefan Hildenbrand and Team UC"
	date: "$Date: 2007-04-13 15:52:10 +0200 (Fri, 13 Apr 2007) $"
	revision: "$Revision: 2562 $"

class
	UC_PUZZLE

create
	make_empty

feature -- initialization
	make_empty is
			-- creates an empty puzzle.
			-- use a cutter to make puzzles
			do
				create part_list.make
			end


feature -- Access

	get_puzzle_part (pos_x: INTEGER; pos_y: INTEGER): UC_PUZZLE_PART is
			-- get the puzzle_part with pos_x and pos_y, Void when not found
		local
			found: BOOLEAN -- has the part been found?
			pp: UC_PUZZLE_PART
		do
			found := False
			from part_list.start
			until found or else part_list.after
			loop
				pp ?= part_list.item_for_iteration
				if pos_x = pp.pos_x and then pos_y = pp.pos_y then
					found := True
					Result := pp
				end
				part_list.forth
			end
		end

	image_width: INTEGER
			-- width of the image, this puzzle was made of

	image_height: INTEGER
			-- height of the image, this puzzle was made of

	part_width: INTEGER
			-- width of one part of this puzzle

	part_height: INTEGER
			-- height of one part of this puzzle

	num_parts_width: INTEGER
			-- number of parts in width

	num_parts_height: INTEGER
			-- number of parts in height		


feature {QUAD_CUTTER} -- Status setting
	set_image_size (size : TUPLE[INTEGER, INTEGER]) is
			-- set size of the image to size=(width, height)
			require
				size_not_void: size /= void
			do
				image_width := size.integer_item (1)
				image_height := size.integer_item (2)
			end

	set_part_size (size : TUPLE[INTEGER, INTEGER]) is
			-- set size of the parts to size=(width, height)
			require
				size_not_void: size /= void
			do
				part_width := size.integer_item (1)
				part_height := size.integer_item (2)
			end

	set_part_num (num : TUPLE[INTEGER, INTEGER]) is
			-- set number of parts to num=(num in width, num in height)
			require
				num_not_void: num /= void
			do
				num_parts_width := num.integer_item (1)
				num_parts_height := num.integer_item (2)
			end

feature {QUAD_CUTTER} -- Element change

	extend (new_part : UC_PUZZLE_PART) is
			-- put new part in part_list
			require
				new_part_not_void: new_part /= void
			do
				part_list.put_right (new_part)
			end

feature -- Element Change
	mix_up is
			-- mixes given list up
			local
				rand_gen : RANDOM_NUMBER_GENERATOR -- random generator
				i : INTEGER -- local variable
				temp_list: like part_list -- temporary list
			do
				create rand_gen.make -- create and initialize random generator
				create temp_list.make -- create result
				from

				until
					part_list.count < 37 -- stop when almost all elements are in the new list
				loop
					i := rand_gen.next_int_mod_minus_1 (37) -- get new random number
					temp_list.put_right(part_list.item(i)) -- put item at random position in new list
					part_list.remove (i) -- remove item that was just moved to new list from old list
				end
				from

				until
					part_list.count < 19 -- stop when almost all elements are in the new list
				loop
					i := rand_gen.next_int_mod_minus_1 (19) -- get new random number
					temp_list.put_right(part_list.item(i)) -- put item at random position in new list
					part_list.remove (i) -- remove item that was just moved to new list from old list
				end
				from

				until
					part_list.count < 5 -- stop when almost all elements are in the new list
				loop
					i := rand_gen.next_int_mod_minus_1 (5) -- get new random number
					temp_list.put_right(part_list.item(i)) -- put item at random position in new list
					part_list.remove (i) -- remove item that was just moved to new list from old list
				end
				temp_list.extend_right(part_list) -- just add the rest of the list at the end
				part_list := temp_list
			end


feature {UC_PUZZLE_VIEW} -- Parts
	part_list: DS_BILINKED_LIST[EM_DRAWABLE]
			-- the list of all puzzle_parts in this puzzle

end -- class UC_PUZZLE

