indexing
	description: "This class denotes the model of the puzzle parts in game UC."
	author: "Stefan Hildenbrand and Team UC"
	date: "$Date: 2007-04-13 15:52:10 +0200 (Fri, 13 Apr 2007) $"
	revision: "$Revision: 2562 $"

class
	UC_PUZZLE_PART

inherit
	EM_DRAWABLE
		redefine default_create, is_equal end
		-- altough a puzzle_part is itself drawable, one should use a uc_puzzle_part_view to actually draw a uc_puzzle_part

	EM_SHARED_BITMAP_FACTORY
		export {NONE} all
		undefine default_create, copy, is_equal end
		-- we need the bitmapfactory to rotate the pictures

create
	make_from_bitmap

feature {NONE} -- Creation
	default_create is
			-- initialize object
			-- DO NOT USE THIS ONE
			do
				Precursor {EM_DRAWABLE}
				bitmap_factory.create_empty_bitmap (10,10)
				picture := bitmap_factory.last_bitmap
			end


	make_from_bitmap(bitmap: EM_BITMAP) is
			-- initialize object
			do
				picture := bitmap
				width := bitmap.width
				height := bitmap.height
				locked := false
			end

feature -- Comparison

	is_equal (other : like current) : BOOLEAN is
			-- tow puzzle_parts are equal, if they show the same part of the picture
			do
				Result := (pos_x = other.pos_x) and (pos_y = other.pos_y)
			end


feature -- Status
	width: INTEGER
			-- width of this part

	height: INTEGER
			-- height of this part


	pos_x : INTEGER
			-- position of this part in the whole puzzle
			-- coords start at top right
			-- pos_x gives col


	pos_y : INTEGER
			-- position of this part in the whole puzzle
			-- coords start at top left
			-- pos_y gives row

	locked: BOOLEAN
			-- may this part be moved, that means, is part already placed correctly ?



	correct(ax,ay:INTEGER): BOOLEAN is
			-- is this part at its correct place ?
			-- x,y denote the top left corner of to_field
			local
				x_correct, y_correct: BOOLEAN
				soll_x, soll_y: INTEGER
			do
				soll_x := ax + (pos_x * width)
				soll_y := ay + (pos_y * height)
				x_correct := (x-soll_x).abs < 20
				y_correct := (y-soll_y).abs < 20
				result := x_correct AND y_correct
			end

feature -- Drawing

	draw(a_surface: EM_SURFACE) is
			-- draw part on a_surface
			-- for compatibility. use a view of this part instead and then call draw_oriented
			do
				picture.set_x_y(x,y)
				picture.draw(a_surface)
			end

	draw_oriented(a_surface: EM_SURFACE; orient,an_x,an_y : INTEGER) is
			-- draw this part on a_surface with given orient
			require
				orient_feasible: (orient>0 and orient<5)
				surface_there: a_surface /= void
			do
				inspect
					orient
				when 2 then
					picture2.set_x_y (an_x, an_y)
					width := picture2.width
					height := picture2.height
					picture2.draw(a_surface)
				when 3 then
					picture3.set_x_y (an_x, an_y)
					width := picture3.width
					height := picture3.height
					picture3.draw(a_surface)
				when 4 then
					picture4.set_x_y (an_x, an_y)
					width := picture4.width
					height := picture4.height
					picture4.draw(a_surface)
				else
					picture.set_x_y (an_x, an_y)
					width := picture.width
					height := picture.height
					picture.draw(a_surface)
				end
			end

feature -- Element change

	lock  is
			-- lock this part
			do
				locked := true
			end




	set_pos (an_x, an_y: INTEGER) is
			-- set pos of this part in the puzzle
			do
				pos_x := an_x
				pos_y := an_y
			end

feature -- Pictures
	picture: EM_BITMAP
		-- the picture of the part


	-- following singleton access to rotated images
	picture2: EM_BITMAP is
		do
			if pic2 = void then
				bitmap_factory.create_rotated_bitmap (picture, 90.0)
				pic2 :=  bitmap_factory.last_bitmap
			end
			Result := pic2
		end

	pic2: EM_BITMAP

	picture3: EM_BITMAP is
		do
			if pic3 = void then
				bitmap_factory.create_rotated_bitmap (picture, 180.0)
				pic3 :=  bitmap_factory.last_bitmap
			end
			Result := pic3
		end

	pic3: EM_BITMAP

	picture4: EM_BITMAP is
		do
			if pic4 = void then
				bitmap_factory.create_rotated_bitmap (picture, 270.0)
				pic4 :=  bitmap_factory.last_bitmap
			end
			Result := pic4
		end

	pic4: EM_BITMAP

invariant
	pic_not_void : picture /= void

end -- class UC_PUZZLE_PART



