indexing
	description: "This is a view of a puzzle_part.."
	author: "Stefan Hildenbrand and Team UC"
	date: "$Date: 2007-04-13 15:52:10 +0200 (Fri, 13 Apr 2007) $"
	revision: "$Revision: 2562 $"

class
	UC_PUZZLE_PART_VIEW

inherit
	EM_DRAWABLE
		redefine is_equal end

create
	make_from_puzzle_part

feature -- Initialization

	make_from_puzzle_part(puzz_part: UC_PUZZLE_PART) is
				-- create a new view of given puzz_part
			require
				puzz_part_not_void: puzz_part /= void
			do
				puzzle_part := puzz_part
				orientation := 1
			end

feature -- Comparison

	is_equal (other: like current) : BOOLEAN is
			-- two puzzle_part_views are equal if they show the same part
			do
				Result := (puzzle_part = other.puzzle_part)
			end


feature -- Status report

	save: STRING is
			-- give all the information that are required to save this ppv
		local
			str: STRING
		do
			create str.make_empty
			str.append_integer (x) -- write the ppv's x-Position to str
			str.append (" ")
			str.append_integer (y) -- write the ppv's y-Position to str
			str.append (" ")
			str.append_integer (orientation) -- write the ppv's orientation to str
			str.append (" ")
			str.append_boolean (locked) -- write whether the ppv is locked or not to str
			str.append (" ")
			str.append_integer (puzzle_part.pos_x) -- write the x-Position of the pp behind the ppv to str
			str.append (" ")
			str.append_integer (puzzle_part.pos_y) -- write the y-Position of the pp behind the ppv to str
			Result := str
		ensure
			Result_not_Void: Result /= Void
		end


	orientation: INTEGER
		-- how is this view oriented, 1 is normal, 2 rotated by 90 deg clockwise usw

	width: INTEGER is
			-- width of this view
			do
				if (orientation=1 or orientation=3) then
					result := puzzle_part.width
				else
					result := puzzle_part.height
				end
			end

	height: INTEGER is
			-- height of this part
			do
				if (orientation=1 or orientation=3) then
					result := puzzle_part.height
				else
					result := puzzle_part.width
				end
			end

	locked: BOOLEAN is
			-- is part locked
			do
				Result := puzzle_part.locked
			end

	correct(ax, ay: INTEGER): BOOLEAN is
			-- is part correctly set
			-- ax, ay denote coordination transformation
			do
				puzzle_part.set_x_y(x,y)
				Result := puzzle_part.correct(ax,ay) AND orientation = 1
			end

feature -- Element change

	rotate is
			-- rotate view
			do
				orientation := (orientation\\4) + 1
			end

	lock is
			-- lock this part
			do
				puzzle_part.lock
			end

feature -- Drawing

	draw(a_surface: EM_SURFACE) is
		-- draw part on a_surface
			do
				puzzle_part.draw_oriented(a_surface, orientation, x, y)
			end

	draw_solved(a_surface: EM_SURFACE; ax,ay : INTEGER) is
			-- draw part on a_surface
			require
				puzzle_part.locked
			do
				puzzle_part.set_x_y(puzzle_part.pos_x * width + ax, puzzle_part.pos_y * height + ay)
				puzzle_part.draw(a_surface)
			end

feature -- Implementation

	puzzle_part: UC_PUZZLE_PART
		-- the part this view shows

end -- class UC_PUZZLE_PART_VIEW



