indexing
	description: "this is a special kind of puzzle_view, which supports sorted insertions, but does not impose restriction on extending or deleting"
	author: "Stefan Hildenbrand and Team UC"
	date: "$Date: 2007-04-13 15:52:10 +0200 (Fri, 13 Apr 2007) $"
	revision: "$Revision: 2562 $"

class
	UC_WORKSPACE

inherit
	UC_PUZZLE_VIEW
		redefine
			draw, delete, width, height
		end

	EM_SHARED_BITMAP_FACTORY
		export {NONE} all
		undefine copy, is_equal
		end

create
	make_from_puzzle, make_from_other, make

feature -- Drawing

	draw (a_surface: EM_SURFACE) is
			-- Draw `Current' to `a_surface'
			-- this draws the parts in this workspace in a sorted way
		local
			cursor: DS_BILINKED_LIST_CURSOR [EM_DRAWABLE]
			translation: EM_VECTOR_2D
			k : INTEGER -- this is a counter
			part_size : INTEGER -- save locally the size of the parts
			old_clipping_area, clipping_area: EM_RECTANGLE
			ppv: UC_PUZZLE_PART_VIEW -- needed for casting
		do
			-- Translate coordinate system for drawing all contained objects.
			create translation.make (x, y)
			a_surface.translate_coordinates (translation)

			-- Change clipping area to clip all objects to container boundaries.
			old_clipping_area := a_surface.coordinate_area
			create clipping_area.make_from_coordinates (0, 0, width, height)
			clipping_area := clipping_area.intersection (old_clipping_area)
			a_surface.clip_coordinates (clipping_area)

			-- set spacing accordingly to part size
			part_size := part_height.max(part_width)

			-- Draw all contained objects.
			cursor := new_cursor
			from
				cursor.start
				k := 0
			until
				cursor.off
			loop
				ppv ?= cursor.item
				if ppv /= void and then ppv.locked then
					-- part is already solved, so do not draw it
					remove_at_cursor (cursor)
				else
					-- part is not solved yet, draw it
					cursor.item.set_x_y ((k\\parts_in_row)*(part_size+10),(k//parts_in_row)*(part_size+10))
					a_surface.draw_object (cursor.item)
					cursor.forth
					k := k+1
				end
			end

			-- Reset coordinate system.			
			a_surface.clip_coordinates (old_clipping_area)
			a_surface.translate_coordinates (- translation)
		end

feature -- Status Report

	save: STRING is
			-- give a string back that holds all information for saving this workspace
		local
			str: STRING -- string to be resulted
			ppv: UC_PUZZLE_PART_VIEW -- puzzle part view that actually will be saved
		do
			create str.make_empty
			from start
			until after
			loop
				ppv ?= item_for_iteration
--				str.append_integer (index) -- write the index of the ppv in the i-th workspace to the file,   OBSOLETE
--				str.append (" ")
				str.append (ppv.save) -- all the information from the ppv to the file
				str.append ("%N") -- put a new line at the end
				forth
			end
			Result := str
		ensure
			Result_not_Void: Result /= Void
		end

	width: INTEGER is
			-- width of this workspace
			do
				Result := parts_in_row * (part_width.max(part_height)+10)
			end

	height: INTEGER is
			-- height of this workspace
			do
				Result := (count/parts_in_row).ceiling* (part_width.max(part_height)+10)
			end

feature -- Element change

	hidden_part: UC_PUZZLE_PART_VIEW
		-- remember the part we hide

	hidden_index: INTEGER
		-- where is the part we had hidden

	hide (part_view : UC_PUZZLE_PART_VIEW) is
			-- hide given part, but do not reorder parts yet
			require
				item_is_there: has(part_view)
			do
				start -- move internal cursor to start
				hidden_part := part_view -- save the part to hide
				search_forth (part_view) -- serach the part
				hidden_index := index -- save the index of the part
				bitmap_factory.create_empty_bitmap (0,0) -- create dummy element
				replace_at(bitmap_factory.last_bitmap) -- put dummy in list
			end

	unhide is
			-- show the part that was hidden before
			-- this function si tolerant, if there is no part we hide, just do nothing
			--require
			--	hidden_part_exists: hidden_part /= void
			do
				if hidden_part /= void then
					replace (hidden_part, hidden_index)
					hidden_part := void
				end
			end

	cond_delete(v: EM_DRAWABLE) is
			-- sometimes we do want to make an conditional delete
			require
				v_not_void: v/= void
			do
				delete(v)
				-- but in the usual case we just want to delete
			end


	delete(v: EM_DRAWABLE) is
			-- redefine delete to handle the case where we want to delete
			-- the hidden element (which will most likely happen very often)
			do
				if v = hidden_part then
					remove (hidden_index)
					hidden_part := void
				else
					Precursor (v)
				end
			end

	extend_sorted(v: UC_PUZZLE_PART_VIEW) is
			-- put given part v in the list, such that it appears there, where the user moved it
			-- ax,ay are coordinates, the source is the top right corner of this container!
			require
				v_not_void: v /= void
			local
				temp_x, temp_y : INTEGER -- temporary coordinates used for calculating
				size : INTEGER -- the space between two parts
				row, col : INTEGER -- place of the part to extend
				idx : INTEGER -- index of the part to place
			do
				-- transform coordinates first
				size := part_height.max(part_width)
				temp_x := v.x - x + size//2
				temp_y := v.y -  y + size//2

				col := ((temp_x - (size//2))  // (size+10))+ 1
				row := (temp_y-5) // (size+10)

				-- transform col and row in an index
				idx := (row * parts_in_row ) + col

				if idx < 1 then
					idx := 1
				elseif idx >= count then
					idx := count
				end

				-- put it at calculated index
				put (v, idx)
			end

	sortable (v : UC_PUZZLE_PART_VIEW) : BOOLEAN is
			-- is a part with given coordinates sortable in this workspace
			require
				v_not_void: v /= void
			do
				result := (v.x > x-10) and (v.x < x+10+width) and  (v.y > y-10) and (v.y < y+10+height)
			end

	set_list(new_list : DS_BILINKED_LIST[EM_DRAWABLE]) is
			-- copy given part_list to this workspace
			require
				argument_not_void : new_list /= void
			do
				wipe_out
				-- delete elements in this workspace
				extend_right (new_list)
				-- and put the elements from given list in it
			end


end -- class UC_WORKSPACE

