indexing
	description: "[
					Keyframe objects represent a certain point in time. They include
					information about 2D position, rotation, scaling and shape of
					the animated object.
					
					All time values are represented in ticks (milliseconds)
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"
				  
	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"


class
	KEYFRAME
	
create
	make_with_time

feature -- Initialization

	make_with_time (a_time: INTEGER) is
			-- Create new keyframe at specified time
		do
			make_with_time_and_position (a_time, 0, 0)
		end
		
	make_with_time_and_position (a_time: INTEGER; an_x, an_y: INTEGER) is
			-- Create new keyframe at specified time and position
		do
			time := a_time
			set_position (an_x, an_y)
			set_rotation (0)
			set_scaling (1.0)
			set_cycle_base_image (0)
			set_cycle_image_count (1)
			set_cycle_phase_interval (0)
		end

feature -- Status

	time: INTEGER

	set_position (an_x, an_y: INTEGER) is
			-- Set horizontal and vertical position
		do
			x := an_x
			y := an_y
		end
		
	x: INTEGER
	y: INTEGER
	
	set_rotation (an_angle: INTEGER) is
			-- Set angle in degrees
		do
			rotation := an_angle
		end
		
	rotation: INTEGER
	
	set_scaling (a_factor: DOUBLE) is
			-- Set scaling factor
		do
			scaling := a_factor
		end
		
	scaling: DOUBLE
	
	set_image_path (a_path: STRING) is
			-- Set path of image used between current and following keyframe
			-- If void, current image remains unchanged
		do
			image_path := a_path
		end
		
	image_path: STRING
	
	set_cycle_base_image (a_value: INTEGER) is
			-- Set the number of the first image used by the current cycle
		do
			cycle_base_image := a_value
		end
		
	cycle_base_image: INTEGER
		-- Number of first image used by the current cycle
		
	set_cycle_image_count (a_value: INTEGER) is
			-- Set the number of images used by the current cycle
		do
			cycle_image_count := a_value
		end
		
	cycle_image_count: INTEGER
		-- Number of images used by current cycle
		
	set_cycle_phase_interval (an_interval: INTEGER) is
			-- Set interval of phase change in milliseconds
		do
			cycle_phase_interval := an_interval
		end
		
	cycle_phase_interval: INTEGER

end -- class KEYFRAME
