indexing
	description: "[
					The singleton object of GUI_CONSTANTS encapsulates various constants
					such as colors, fonts, default image paths and size parameters.
					It also provides a 'ticks' function (not exactly a constant) which
					gives you the number of milliseconds since the app was started.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"

	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_CONSTANTS

inherit
	EM_TIME_SINGLETON
	export {NONE} all end

create {GUI_SHARED}
	default_create

feature -- Colors

	color_transparent: EM_COLOR is
			-- transparent color
		once
			result := create {EM_COLOR}.make_black
			result.set_alpha (0)
		end

	color_black: EM_COLOR is
			-- black color
		once
			result := create {EM_COLOR}.make_black
		end

	color_dark_grey: EM_COLOR is
			-- dark grey color
		once
			result := create {EM_COLOR}.make_with_rgb (64, 64, 64)
		end

	color_light_grey: EM_COLOR is
			-- light grey color
		once
			result := create {EM_COLOR}.make_with_rgb (192, 192, 192)
		end

	color_white: EM_COLOR is
			-- black color
		once
			result := create {EM_COLOR}.make_white
		end

	color_yellow: EM_COLOR is
			-- yellow color
		once
			result := create {EM_COLOR}.make_with_rgb (255, 255, 0)
		end

	color_closeup_fill: EM_COLOR is
			-- color for closeup fill
		once
			Result := create {EM_COLOR}.make_with_rgb (0, 0, 0)
			Result.set_alpha (170)
		end

	color_closeup_border_normal: EM_COLOR is
			-- color for closeup border in normal state
		once
			Result := create {EM_COLOR}.make_with_rgb (164, 45, 45)
		end

	color_closeup_border_hoover: EM_COLOR is
			-- color for closeup border in hoover state
		once
			Result := create {EM_COLOR}.make_with_rgb (212, 52, 52)
		end

	color_option_fill_normal: EM_COLOR is
			-- color for option text fill
		once
			Result := clone (color_black)
			Result.set_alpha (192)
		end

	color_option_fill_hoover: EM_COLOR is
			-- color for option text fill
		once
			Result := create {EM_COLOR}.make_with_rgb (212, 52, 52)
			Result.set_alpha (128)
		end

	color_menu_border_normal: EM_COLOR is
			-- color for menu borders
		once
			result := create {EM_COLOR}.make_with_rgb (38, 80, 133)
		end

	color_menu_border_hoover: EM_COLOR is
			-- color for menu border
		once
			Result := create {EM_COLOR}.make_with_rgb (7, 100, 215)
		end


	color_menu_fill: EM_COLOR is
			-- color for menu backgrounds
		once
			result := create {EM_COLOR}.make_with_rgb (9, 30, 61)
		end

	color_repository_border_normal: EM_COLOR is
			-- color for repository border
		once
			result := create {EM_COLOR}.make_with_rgb (38, 80, 113)
		end

	color_repository_border_hoover: EM_COLOR is
			-- color for repository border
		once
			Result := create {EM_COLOR}.make_with_rgb (7, 100, 215)
		end

	color_repository_fill: EM_COLOR is
			-- color for repository background
		once
			result := create {EM_COLOR}.make_with_rgb (9, 30, 61)
		end

	color_button_border_normal: EM_COLOR is
			-- color for button item border
		once
			result := create {EM_COLOR}.make_with_rgb (20, 77, 162)
		end

	color_button_fill_normal: EM_COLOR is
			-- color for button item border
		once
			result := create {EM_COLOR}.make_with_rgb (16, 51, 104)
		end

	color_button_fill_hoover: EM_COLOR is
			-- color for button item background
		once
			result := create {EM_COLOR}.make_with_rgb (37, 112, 205)
		end

	color_button_text_disabled: EM_COLOR is
			-- color for button item background
		once
			result := create {EM_COLOR}.make_with_rgb (100, 100, 128)
		end

	color_item_fill_normal: EM_COLOR is
			-- color for repository item border
		once
			result := create {EM_COLOR}.make_with_rgb (11, 39, 81)
		end

	color_item_fill_hoover: EM_COLOR is
			-- color for repository item background
		once
			result := create {EM_COLOR}.make_with_rgb (22, 89, 172)
		end

	color_item_border_selected: EM_COLOR is
			-- color for selected repository item
		once
			result := create {EM_COLOR}.make_with_rgb (255, 255, 0)
		end

	color_tooltip_border: EM_COLOR is
			-- border color for tooltip
		once
			result := create {EM_COLOR}.make_with_rgb (201, 255, 86)
		end

	color_tooltip_fill: EM_COLOR is
			-- fill color for tooltip
		once
			result := color_black
		end

	color_credits_text_1: EM_COLOR is
			-- color for selected repository item
		once
			result := create {EM_COLOR}.make_with_rgb (212, 255, 138)
		end

	color_credits_text_2: EM_COLOR is
			-- color for selected repository item
		once
			result := create {EM_COLOR}.make_with_rgb (94, 237, 144)
		end

	color_credits_text_3: EM_COLOR is
			-- color for selected repository item
		once
			result := create {EM_COLOR}.make_with_rgb (47, 220, 198)
		end

	color_credits_text_4: EM_COLOR is
			-- color for selected repository item
		once
			result := create {EM_COLOR}.make_with_rgb (2, 24, 70)
		end


feature -- Fonts

	font_normal: GUI_PROPORTIONAL_FONT is
			-- standard font, normal face
		once
			result := create {GUI_PROPORTIONAL_FONT}.make_with_font_resource ("fonts/isonorm_normal_512")
		end

	font_bold: GUI_PROPORTIONAL_FONT is
			-- standard font, bold face
		once
			result := create {GUI_PROPORTIONAL_FONT}.make_with_font_resource ("fonts/isonorm_bold_512")
		end

	font_big: GUI_PROPORTIONAL_FONT is
			-- big font, normal face
		once
			result := create {GUI_PROPORTIONAL_FONT}.make_with_font_resource ("fonts/isonorm_normal_1024")
		end


feature -- Image path

	path_skin: STRING is "images/default_skin.tga"

	path_panorama_default_1: STRING is "images/p_default_1.jpg"
	path_panorama_default_2: STRING is "images/p_default_2.jpg"

feature -- Window size

	window_width: INTEGER is 1024
	window_height: INTEGER is 768
	window_aspect: DOUBLE is 1.3333333

feature -- Time service

	ticks: INTEGER is
			-- get milliseconds since start of esdl
		do
			result := time.ticks
		end

feature -- Panorama

	stage_radius: INTEGER is 650

end -- class GUI_CONSTANTS
