indexing
	description: "[
					Scene displaying credits.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"

	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_CREDITS

inherit
	GUI_SCENE
	redefine
		initialize_scene,
		draw_gl_world,
		handle_menu_event_credits
	end

create
	make

feature -- Initialization

	make is
			-- Create new credits scene
		do
			default_create
			create clickables.make
			create timers.make
		end

	initialize_scene is
			-- Initialize graphics for displaying the game credits
		do
			-- Set up the projection perspective
			macro_set_projection_perspective

			-- Register the event handlers
			register_event_handlers

			-- Create tooltip
			create tooltip.make_with_text ("Tooltip", gui_constants.font_normal)
			tooltip.set_invisible

			-- Various settings
			angle := 0
			drawing_mode := drawing_mode_world

			credits_scene_count := credits_texts.count // 2
			game_skin.main_menu.button_credits.set_text ("Continue")

			reset_scene_time

			-- Disable main menu buttons
			game_skin.main_menu.disable_except (game_skin.main_menu.button_credits)

			-- CHANGES event loop
			create event_loop.make_poll
			create animation_event

		end

feature -- Status

	set_previous_game_scene (a_scene: SCENE) is
			-- Set scene where execution should return to after
			-- displaying the credits
		do
			previous_game_scene := a_scene
		end

	previous_game_scene: SCENE

feature -- Operation

	draw_gl_world is
			-- Display the game credits according to given time
		local
			font_1, font_2: GUI_PROPORTIONAL_FONT
			angle1, angle2, spacing: DOUBLE
			cycle_time: DOUBLE
			cycle: INTEGER
			x1, x1b, x1c, x1d, x2, x2b, x2c, x2d, y: DOUBLE
		do
			font_1 := gui_constants.font_big
			font_2 := gui_constants.font_big
			if not scene_in_transition then
				scene_time := gui_constants.ticks - scene_start
			end
			cycle_time := (scene_time \\ credits_interval) / credits_interval
			cycle := (scene_time // credits_interval) \\ credits_scene_count

			angle1 := cycle_time * 65 - 50
			angle2 := cycle_time * 75 - 30
			spacing := gui_timing.exp_bound ((cycle_time - 0.6) * 2.5) * 150

			x1 := -800 * gui_timing.smooth_bound_inv (cycle_time * 8 + 0.2) + cycle_time *  200 - 200
			x1b := x1  + cycle_time * 500 - 250
			x1c := x1b + cycle_time * 500 - 250
			x1d := x1  + cycle_time * 700 - 350
			x2 :=  1000 * gui_timing.smooth_bound_inv (cycle_time * 8 - 0.35) + cycle_time * -100 + 150
			x2b := x2  + 250 - cycle_time * 500
			x2c := x2b + 250 - cycle_time * 500
			x2d := 300 * gui_timing.smooth_bound_inv (cycle_time * 8 - 0.35) + x2 + 100 - cycle_time * 200

			y := 0
			emgl_clear (EM_gl_color_buffer_bit | EM_gl_depth_buffer_bit)

			emgl_load_identity

			emgl_translatef (-20, 100, -630)
			emgl_rotatef (12, 0, 0, 1)
			emgl_rotatef (angle2, 1, 0, 0)
			emgl_rotatef (angle1, 0, 1, 0)

			emgl_push_matrix
			emgl_rotatef (45, 0, 1, 0)
			emgl_rotatef (180, 1, 0, 0)
			emgl_translatef (0, 0, 300)
			font_1.set_alignment_centered
			font_1.set_spacing_offset (spacing * 5)


			-- CHANGES
			-- font_1.set_gl_scaling (5)
			font_1.set_gl_scaling(1)

			font_1.set_text_color (gui_constants.color_credits_text_4)
			font_1.draw_text_line (credits_texts.item (cycle * 2 + 1), gui.screen, x1d, -300 + y)

			font_1.set_spacing_offset (spacing)
			emgl_translatef (0, 0, -100)
			font_1.set_gl_scaling (0.5)
			font_1.set_text_color (gui_constants.color_credits_text_3)
			font_1.draw_text_line (credits_texts.item (cycle * 2 + 1), gui.screen, x1c, -30 + y)

			emgl_translatef (0, 0, -100)
			font_1.set_gl_scaling (0.7)
			font_1.set_text_color (gui_constants.color_credits_text_2)
			font_1.draw_text_line (credits_texts.item (cycle * 2 + 1), gui.screen, x1b, -30 + y)

			emgl_translatef (0, 0, -100)
			font_1.set_gl_scaling (0.9)
			font_1.set_text_color (gui_constants.color_credits_text_1)
			font_1.draw_text_line (credits_texts.item (cycle * 2 + 1), gui.screen, x1, -30 + y)

			emgl_pop_matrix
			emgl_rotatef (-45, 0, 1, 0)
			emgl_rotatef (180, 1, 0, 0)
			emgl_translatef (0, 0, 300)
			font_2.set_alignment_centered
			font_2.set_spacing_offset (spacing * 5)


			-- CHANGES
			-- font_2.set_gl_scaling (5)
			font_2.set_gl_scaling(1)

			font_2.set_text_color (gui_constants.color_credits_text_4)
			font_2.draw_text_line (credits_texts.item (cycle * 2 + 2), gui.screen, x2d, 110 + y)

			font_2.set_spacing_offset (spacing)
			emgl_translatef (0, 0, -100)
			font_2.set_gl_scaling (0.5)
			font_2.set_text_color (gui_constants.color_credits_text_3)
			font_2.draw_text_line (credits_texts.item (cycle * 2 + 2), gui.screen, x2c, 30 + y)

			emgl_translatef (0, 0, -100)
			font_2.set_gl_scaling (0.8)
			font_2.set_text_color (gui_constants.color_credits_text_2)
			font_2.draw_text_line (credits_texts.item (cycle * 2 + 2), gui.screen, x2b, 30 + y)

			emgl_translatef (0, 0, -100)
			font_2.set_gl_scaling (1.1)
			font_2.set_text_color (gui_constants.color_credits_text_1)
			font_2.draw_text_line (credits_texts.item (cycle * 2 + 2), gui.screen, x2, 30 + y)

		end

feature {GUI_CLICKABLE} -- Event handling

	handle_menu_event_credits is
			-- Button 'credits' is selected from main menu
		require else
			previous_game_scene_valid: previous_game_scene /= Void
		local
			transition: GUI_TRANSITION_CUBE
		do
			game_skin.main_menu.button_credits.set_text ("Credits")

			prepare_scene_transition
			if previous_game_scene = Void then
				next_scene := create {GUI_DEFAULT_SCENE}.make_default_scene
			else
				next_scene := create {GUI_GAME_SCENE}.make_from_game_scene (previous_game_scene)
			end
			next_scene.initialize_scene

			create transition
			transition.do_transition (Current, next_scene, screen)
		end

feature {NONE} -- Implementation


	credits_scene_count: INTEGER

	credits_interval: INTEGER is 4500

	credits_texts: ARRAY [STRING] is
			-- Text resource for credits
		once
			Result := << "XAE", "Adventure Engine",
						  "Graphics & Design", "Ralph Wiedemeier",
						  "Game Engine", "Adrian Rabenseifner",
						  "XML Parser", "Fabrizio Steiner",
						  "Persistent Storage", "Philipp Wolfensperger",
						  "Director of Photography", "Ralph Wiedemeier",
						  "Storyboard", "Adrian Rabenseifner",
						  "XML Programming", "Fabrizio Steiner",
						  "Location Scout", "Philipp Wolfensperger",
						  "Support Assistance", "Stephanie Balzer",
						  "ESDL Library", "Till G. Bay",
						  "Thanks to Mr. Eiffel", "Bertrand Meyer",
						  "Copyright 2005", "All rights reserved" >>
		end


end -- class GUI_CREDITS
