indexing
	description: "[
					Objects of class GUI_FONT_FACE provide a convenient way
					to manage attributes of a GUI_PROPORTIONAL_FONT.
					representing the graphical user interface.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"
				  
	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_FONT_FACE

create
	make_with_font


feature -- Initialization

	make_with_font (a_font: GUI_PROPORTIONAL_FONT) is
			-- Create new font face using the specified bitmap font
		do
			set_font (a_font)
		end
		
feature -- Status

	set_font (a_font: GUI_PROPORTIONAL_FONT) is
			-- Set bitmap font used for this font face
		require
			valid_font: a_font /= Void
		do
			font := a_font
		end
		
	font: GUI_PROPORTIONAL_FONT

	set_gl_scaling (a_value: DOUBLE) is
			-- Set scaling factor.
			-- Scaling on fonts only takes effect in gl mode.
		require
			factor_valid: a_value > 0.2 and then a_value <= 2
		do
			gl_scaling := a_value
		end
		
	gl_scaling: DOUBLE

	set_spacing_offset (an_offset: DOUBLE) is
			-- set spacing offset in pixels
		do
			spacing_offset := an_offset
		end
		
	spacing_offset: DOUBLE
	
	set_line_spacing (a_spacing: DOUBLE) is
			-- set line spacing
		do
			line_spacing := a_spacing
		end
		
	line_spacing: DOUBLE
	
	set_max_characters (a_number: INTEGER) is
			-- Set maximum number of displayed characters
		do
			if a_number >= 0 then
				max_characters := a_number
			else
				max_characters := 0
			end
		end
		
	max_characters: INTEGER
	
	set_alignment_left is
			-- left align text in framed output
		do
			alignment := align_left
		end
		
	set_alignment_centered is
			-- center text in framed output
		do
			alignment := align_center
		end
		
	set_alignment_right is
			-- right align text in framed output
		do
			alignment := align_right
		end
		
	set_text_color (a_color: ESDL_COLOR) is
			-- set text color
		do
			text_color := a_color
		end
		
	text_color: ESDL_COLOR

feature -- Drawing

	draw_text_multiline (a_string: STRING; a_box: ESDL_RECT; a_surface: ESDL_SURFACE) is
			-- Draws 'a_string' to 'a_surface' into the specified box
			-- using automatic word wrapping
		do
			apply_font_attributes
			font.draw_text_multiline (a_string, a_box, a_surface)
		end

	draw_text_line (a_string: STRING; a_surface: ESDL_SURFACE; x, y: DOUBLE) is
			-- Draws 'a_string' to 'a_surface' at the specified position
			-- using the attributes of current font face
		do
			apply_font_attributes
			font.draw_text_line (a_string, a_surface, x, y)
		end

	draw_character (a_character: CHARACTER; a_surface: ESDL_SURFACE; x, y: DOUBLE) is
			 -- Draws 'a_character' to 'a_surface' at the specified position
			 -- using the attributes of current font face
		do
			apply_font_attributes
			font.draw_character (a_character, a_surface, x, x)
		end

feature {NONE} -- Implementation

	apply_font_attributes is
			-- Apply attributes of current to the attached bitmap font
		do
			font.set_gl_scaling (gl_scaling)
			font.set_spacing_offset (spacing_offset)
			font.set_line_spacing (line_spacing)
			font.set_max_characters (max_characters)
			font.set_text_color (text_color)
			
			if alignment = align_left then
				font.set_alignment_left
			elseif alignment = align_center then
				font.set_alignment_centered
			else
				font.set_alignment_right
			end
		end
		
	alignment:    INTEGER
		-- information about how framed text should be aligned
	align_left:   INTEGER is 0
	align_center: INTEGER is 1
	align_right:  INTEGER is 2
		-- alignment options
	

end -- class GUI_FONT_FACE
