indexing
	description: "[
					GUI_TIMING is a singleton helper class which provides
					various interpolation smoothing functions. The provided
					functions mainly base on bernstein polynomials.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"
				  
	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

class
	GUI_TIMING

create {GUI_SHARED}
	default_create


feature -- Operations

	smooth (t: DOUBLE): DOUBLE is
			-- Get smooth transition (0..1)
		do
			result := 3*t*t - 2*t*t*t
		end
		
	smooth_bound (t: DOUBLE): DOUBLE is
			-- Get smooth transition (0..1), bounds are checked!
		do
			if t < 0 then
				Result := 0
			elseif t > 1 then
				Result := 1
			else
				Result := 3*t*t - 2*t*t*t
			end
		end

	smooth_bound_inv (t: DOUBLE): DOUBLE is
			-- Get smooth transition (0..1), bounds are checked!
		do
			if t < 0 then
				Result := 1
			elseif t > 1 then
				Result := 0
			else
				Result := 1 - 3*t*t + 2*t*t*t
			end
		end
		
	exp_bound (t: DOUBLE): DOUBLE is
			-- Get smooth transition (0..1), bounds are checked!
		do
			if t < 0 then
				Result := 0
			elseif t > 1 then
				Result := 1
			else
				Result := t*t*t
			end
		end
		
	super_smooth (t: DOUBLE): DOUBLE is
			-- Get super smooth transition (0..1)
		local
			s: DOUBLE
		do
			s := t*t*t
			result := 10*s - 15*s*t + 6*s*t*t
		end
		
	smooth_zero (t: DOUBLE): DOUBLE is
			-- From zero to one to zero (0..1)
		do
			result := 10*t*t - 20*t*t*t + 10*t*t*t*t
		end

end -- class GUI_TIMING
