indexing
	description: "[
					Base class for transition effects.
					
					Such effects can display various transitions from a source
					scene to a destination scene.
					
					Implementations of transition effects should inherit from this
					deferred base class.
					
					This class is part of the XAE Extended Adventure Engine Project.
				  ]"

	author: "Ralph Wiedemeier, ralphw@student.ethz.ch"
	date: "$Date: 2007-02-05 18:24:28 +0100 (Mon, 05 Feb 2007) $"
	revision: "$Revision: 2485 $"

deferred class
	GUI_TRANSITION

inherit
	GUI_SHARED
		export {NONE} all end

	OPEN_GL_LIBRARY
		--export {NONE} all end

--	SDL_SURFACE_STRUCT
--		rename
--			w as width,
--			h as height
--		export
--			{NONE} all
--			{, EM_BITMAP_FACTORY, EM_APPLICATION} item
--			{ANY} pixels
--			{ANY} format
--			{ANY} width
--			{ANY} height
--		end

--	SDL_FUNCTIONS_EXTERNAL
--		export {NONE} all end

--	SDL_VIDEO_FUNCTIONS_EXTERNAL
--		export {NONE} all end

--	SDL_GFXPRIMITIVES_FUNCTIONS
--		export {NONE} all end

--	GL_FUNCTIONS_EXTERNAL
--		export {NONE} all end

--	GLU_FUNCTIONS_EXTERNAL
--		export {NONE} all end

feature -- Initialization

	initialization is
			-- Common initialization tasks
		do
			if duration = 0 then
				duration := default_duration
			end
		end

feature -- Operation

	do_transition (source: GUI_SCENE; destination: GUI_SCENE; surface: EM_VIDEO_SURFACE) is
			-- Perform transition effect from source to destination scene
		require
			valid_source: source /= Void
			valid_destination: destination /= Void
			valid_surface: surface /= Void
		deferred
		end

feature -- Status

	set_duration (a_duration: INTEGER) is
			-- set duration of transition effect in milliseconds
		do
			if a_duration > 0 then
				duration := a_duration
			else
				duration := default_duration
			end
		end

	duration: INTEGER

	set_center (an_x: INTEGER; an_y: INTEGER) is
			-- set center of transition effect
			-- some transitions may not use this parameter
		do
			center_x := an_x
			center_y := an_y
		end

	center_x: INTEGER
	center_y: INTEGER

feature {NONE} -- Implementation

	make_snapshots (source: GUI_SCENE; destination: GUI_SCENE) is
			-- Store snapshots of source and destination scene
			-- in 'fx_texture_1' and 'fx_texture_2'
		require
			valid_source: source /= Void
			valid_destination: destination /= Void
		do
			source.draw_gl_world
			macro_make_snapshot (fx_texture_1)
			destination.draw_gl_world
			macro_make_snapshot (fx_texture_2)
		end

	default_duration: INTEGER is deferred end
			-- Default duration for transition effect

--feature -- CHANGES
--		gl_enter_2d(width, height: INTEGER) is
--			-- Enter 2d mode using opengl.
--		do
--			if
--				not gl_2d_mode
--			then
--				gl_push_attrib_external (em_gl_enable_bit | em_gl_color_buffer_bit )
--				gl_disable_external (em_gl_depth_test )
--				gl_disable_external (em_gl_cull_face )
--				gl_enable_external (em_gl_texture_2d )
--				gl_clear_color_external ( 0, 0, 0, 1 )

--				  -- This allows alpha blending of 2D textures with the scene
--					gl_enable_external (em_gl_blend)
--					gl_blend_func_external (em_gl_src_alpha, em_gl_one_minus_src_alpha)

--				gl_viewport_external ( 0, 0, width, height )

--				-- Set perspective
--				gl_matrix_mode_external (em_gl_projection )
--				gl_push_matrix_external
--				gl_load_identity_external

--				gl_ortho_external ( 0, width, height, 0, 0, 1 )

--				gl_matrix_mode_external (em_gl_modelview )
--				gl_push_matrix_external
--				gl_load_identity_external

--				gl_tex_envf_external (em_gl_texture_env, em_gl_texture_env_mode, em_gl_replace)

--				gl_2d_mode := true
--			end
--		ensure
--			in_mode: gl_2d_mode
--		end

--	gl_leave_2d is
--			-- Leave 2d mode using opengl
--		do
--			if
--				gl_2d_mode
--			then
--				gl_matrix_mode_external ( em_gl_projection )
--				gl_pop_matrix_external

--				gl_matrix_mode_external ( em_gl_modelview )
--				gl_pop_matrix_external

--				gl_pop_attrib_external
--				gl_2d_mode := false
--			end
--		ensure
--			not_in_mode: not gl_2d_mode
--		end


--		gl_2d_mode: BOOLEAN
--			-- Are we in the 2d mode using opengl?	
--		surface_operation_successful: BOOLEAN

--		surf_redraw(surf: ) is
--			-- Redraw `Current' on display.
--		local
--			i: INTEGER
--		do
--			-- if openGL ist enabled and this is the screen surface, use sdl_gl_swap_buffers, else sdl_flip
--			if
--				surf.video_subsystem.opengl_enabled
--					and surf.video_subsystem.video_surface = Current
--			then
--				surf.sdl_gl_swap_buffers_external
--				surface_operation_successful := True -- TODO: Error checking/propagating ??
--			else
--				i := sdl_flip_external (item)
--				if
--					i < 0
--				then
--					last_error := Error_redraw_surface
--					surface_operation_successful := False
--				else
--					surface_operation_successful := True
--				end
--			end
--		ensure
--			surface_operation_successful: surface_operation_successful or else (last_error /= Void and then not last_error.is_empty)
--		end


--redraw is
--			-- Redraw `Current' on display.
--		local
--			i: INTEGER
--		do
--			-- if openGL ist enabled and this is the screen surface, use sdl_gl_swap_buffers, else sdl_flip
--			if
--				video_subsystem.opengl_enabled
--					and video_subsystem.video_surface = Current
--			then
--				sdl_gl_swap_buffers_external
--				surface_operation_successful := True -- TODO: Error checking/propagating ??
--			else
--				i := sdl_flip_external (item)
--				if
--					i < 0
--				then
--					last_error := Error_redraw_surface
--					surface_operation_successful := False
--				else
--					surface_operation_successful := True
--				end
--			end
--		ensure
--			surface_operation_successful: surface_operation_successful or else (last_error /= Void and then not last_error.is_empty)
--		end


end -- class GUI_TRANSITION
