indexing
	description: "[
		A quaternion that can be interpolated between a sequence of control quaternions.
		The generic parameter specifies the interpolation type.
		]"
	date: "$Date: Jan 6, 2007 10:04:18 AM$"
	revision: "$Revision: 1.0$"

class EM_INTERPOLATION_QUATERNION_REF[ G->EM_QUATERNION_INTERPOLATOR create default_create end ]

inherit
	EM_QUATERNION_REF
		redefine
			to_reference
		end

create
	make,
	default_create,
	make_from_rotation,
	make_from_matrix44,
	make_from_arc

feature -- Initialization
	make_from_quaternion( quaternion: EM_QUATERNION ) is
			-- Creates from a reference
		do
			v := quaternion.v
			w := quaternion.w
		end

feature {NONE} -- Interpolator
	internal_interpolator: G
	interpolator: G is
			-- The interpolator uses an interpolation described by G
			-- to interpolate the current quaternion
		do
			if internal_interpolator = void then
				create internal_interpolator
			end
			result := internal_interpolator
		end
feature -- Interpolation functions
	animation_position: DOUBLE
	interpolate_to ( a_quaternion: EM_QUATERNION ) is
			-- Interpolate from the current position to 'a_quaternion'
		do
			interpolator.set_sequence( << to_quaternion, a_quaternion >> )
			animation_position := 0
		end
	interpolate_through ( some_quaternions: ARRAY[EM_QUATERNION] ) is
			-- Interpolate through 'some_quaternions' NOT starting at the
			-- current quaternion
		require
			valid_array: some_quaternions/=void
			array_has_elements: some_quaternions.count > 0
		do
			interpolator.set_sequence( some_quaternions )
			animation_position := 0
			make_from_quaternion( some_quaternions.item ( some_quaternions.lower ) )
		end
	step( an_offset: DOUBLE ) is
			-- Step the interpolation forward by 'an_offset' (relative)
		require
			valid_offset: is_valid_animation_position( animation_position+an_offset )
		do
			animation_position := animation_position+an_offset
			make_from_quaternion( interpolator.interpolate( animation_position ) )
		end
	move( a_position: DOUBLE ) is
			-- Move the interpolation to 'a_position' (absolute)
		require
			valid_offset: is_valid_animation_position( a_position )
		do
			animation_position := a_position
			make_from_quaternion( interpolator.interpolate( a_position ) )
		end
	is_valid_animation_position( t: DOUBLE ): BOOLEAN is
			-- Is 't' a valid animation position?
		do
			result := interpolator.is_valid_animation_position ( t )
		end

feature -- Conversion
	to_reference: EM_INTERPOLATION_QUATERNION_REF[ G ] is
			-- Associated reference of Current
		do
			create Result.make(v,w)
		end
	to_quaternion: EM_QUATERNION is
			-- Assiciated quaternion of Current
		do
			Result.set(v,w)
		end


end -- class EM_INTERPOLATION_QUATERNION_REF
