indexing
	description: "[
			A spherical cubic interpolator [Spline].
			It's used to interpolate smoothly through several control points
		]"
	date: "$Date: Jan 7, 2007 12:58:54 AM$"
	revision: "$Revision: 1.0$"

class EM_QUATERNION_SQUAD

inherit
	EM_QUATERNION_INTERPOLATOR
		redefine
			interpolate,
			set_sequence
		end
	DOUBLE_MATH
		export {NONE} all end

feature {NONE} -- Implementation
	slerp( q1, q2: EM_QUATERNION; t: DOUBLE ): EM_QUATERNION is
			-- Spherial linear interpolation
		require
			valid_t: 0<=t and t<=1
		local
			a, sa: DOUBLE
		do
			a := q1.cutting_angle( q2 ).abs()
			sa := sine(a)
			if sa<0.05 then
				result := q1.scaled( 1-t )+q2.scaled( t )
			else
				sa := 1 / sa
				result := q1.scaled( sine((1-t)*a)*sa )+q2.scaled( sine(t*a)*sa )
			end
		end
	ctrl_points: ARRAY[ EM_QUATERNION ]
feature -- Implementation
	set_sequence (a_sequence: like sequence) is
			-- Set `sequence' to `a_sequence'.
		local
			i: INTEGER
			a: EM_QUATERNION
		do
			Precursor( a_sequence )
			create ctrl_points.make(a_sequence.lower, a_sequence.upper)
			from i:=a_sequence.lower+1 until i>=a_sequence.upper loop
				a := a_sequence.item(i).inversed
				a.make_from_exp(-((a*a_sequence.item(i-1)).log+(a*a_sequence.item(i+1)).log).scaled(1/4))
				ctrl_points.put( a_sequence.item(i)*a, i )
				i := i + 1
			end
		end

	interpolate( t: DOUBLE ): EM_QUATERNION is
			-- Interpolate linear
		local
			tt: DOUBLE
		do
			tt := t-t.floor
			result := slerp(slerp(sequence.item(t.floor + sequence.lower),
								sequence.item(t.ceiling + sequence.lower),tt),
							slerp(ctrl_points.item(t.floor + sequence.lower),
								ctrl_points.item(t.ceiling + sequence.lower),tt),
							2*tt*(1-tt) )
		end
end -- class EM_QUATERNION_SQUAD
