indexing

	description: "[

		A scene which uses OpenGL to draw its content.
		
		Inherit from this class and put your OpenGL drawing code in the feature `draw3d'
		for your normal 3d stuff and in the feature `draw2d' for 2d drawing (e.g. menu system)

		The orthogonal projection for `draw2d' is set up to match the rest of the EM
		drawing system, with (0,0) being the upper-left corner
		(screen_width, screen_height) the lower-right corner.

		See also EM3D_SCENE
	]"
	date: "$Date: 2007-07-11 16:03:30 +0200 (Wed, 11 Jul 2007) $"
	revision: "$Revision: 2609 $"

deferred class EM3D_ORTHO_SCENE

inherit

	EM3D_SCENE
		rename
			draw as draw3d
		redefine
			initialize_scene,
			opengl_draw
		end

feature {EM3D_ORTHO_SCENE} -- Initialization
	initialize_scene is
			-- Update near / far distance
		do
			Precursor
			ortho_z_near := -100
			ortho_z_far := 100
		end

feature -- Access

	ortho_z_near: DOUBLE
			--- See ortho_z_far

	ortho_z_far: DOUBLE
			-- Specify the distances to the nearer and farther depth clipping planes.
			-- These values are negative if the plane is to be behind the viewer.

feature -- Drawing

	opengl_draw is
			-- Do the actual drawing. Dont overwrite this function
			-- unless you want to flush the buffers on you own
		do
			-- 3d stuff goes here:
			Precursor
			-- draw 2d stuff also:
			opengl_draw2d
		end

	opengl_draw2d is
			-- Setup orthogonal projection
			-- (0,0,ortho_z_near) is upper-left-near corner and
			-- (w,h,ortho_z_far) is lower-right-far corner of view frustum
			-- with w := Video_subsystem.video_surface_width
			-- and  h := Video_subsystem.video_surface_height
		local
			w, h: DOUBLE
		do
			w := Video_subsystem.video_surface_width
			h := Video_subsystem.video_surface_height

			-- Setup orthographic projection
			emgl_matrix_mode (Em_gl_projection)
			emgl_push_matrix
			emgl_load_identity
			emgl_ortho (0, w, h, 0, ortho_z_near, ortho_z_far)

			-- Clear modelview matrix
			emgl_matrix_mode (Em_gl_modelview)
			emgl_load_identity

			draw2d (w, h)

			-- Restore projection matrix
			emgl_matrix_mode (Em_gl_projection)
			emgl_pop_matrix

			-- Switch back to modelview
			emgl_matrix_mode (Em_gl_modelview)
		end

	draw2d (screen_width, screen_height: DOUBLE) is
			--
		deferred
		end

end
