indexing

	description: "[

		A scene which uses OpenGL to draw its content.
		
		Inherit from this class and put your OpenGL drawing code in the feature `draw'.
		Overwrite predraw for all the tasks before drawing, and postdraw for those after.  
		Overwrite set_default_settings to change some or all the default settings.
		
		Have a look at the code of `redraw' and `initialize_scene' to see which options
		of OpenGL are alredy set.
		
		To react on input events, redefine the various `handle_*' features.

	]"
	date: "$Date: 2006/07/09 12:44:24 $"
	revision: "$Revision: 1.0 $"

deferred class EM3D_SCENE

inherit

	EM_SCENE
		redefine
			make_scene
		end

	EM_SHARED_SUBSYSTEMS
		export {NONE} all end

	EM_CONSTANTS
		export {NONE} all end

	EMGL_DRAW
		export {NONE} all end

	EMGL_SETTINGS
		export {NONE} all end

	EMGL_VIEW
		export {NONE} all end

	EMGLU_VIEW
		export {NONE} all end

	EM_SHARED_ERROR_HANDLER
		export {NONE} all end

feature {NONE} -- Initialisation

	make_scene is
			-- Initialise default values.
		do
			Precursor {EM_SCENE}
		end

	initialize_scene is
			-- Initialize scene.
		do
			create {EM3D_SIMPLE_SCENE_MANAGER}scene_manager.make
			scene_manager.camera.set_aspect ( Video_subsystem.video_surface_width / Video_subsystem.video_surface_height )

			Video_subsystem.video_surface.disable_opengl_blitting

			emgl_enable (em_gl_cull_face )
			emgl_cull_face ( em_gl_back )

			-- Opengl settings
			emgl_enable (Em_gl_depth_test)

			-- Enable antialiasing
			emgl_enable (Em_gl_line_smooth)
			emgl_hint (Em_gl_line_smooth_hint, Em_gl_nicest)

			-- Since we want to use scale, we need to rescale the normals
			emgl_enable( em_gl_rescale_normal )

			-- Setup the model view matrix
			emgl_matrix_mode (Em_gl_modelview)
			emgl_load_identity


			-- Setup the camera:
			-- Place the camera above the object,
			-- look at the center of the scene,
			-- set orientation of camera
			active_camera := scene_manager.camera.create_new_unnamed
			active_camera_node := scene_manager.world.new_child
			active_camera_node.attach_object( active_camera )
		end

feature -- Access

	scene_manager: EM3D_SCENE_MANAGER
			-- The scene manager, that controles this scene

feature -- em3d
	active_camera: EM3D_CAMERA
	active_camera_node: EM3D_SCENE_NODE

feature -- Drawing
	render_state: INTEGER
		-- shows in which renderstate we are
	-- A list of some renderstates
	rs_unknown: INTEGER is 0x0000
	rs_predraw: INTEGER is 0x1000
	rs_draw: INTEGER is 0x2000
	rs_postdraw: INTEGER is 0x4000


	redraw is
			-- Redraw scene.
		do
			scene_manager.animation.play_next
			update_light

			render_state := rs_predraw
			predraw
			render_state := rs_draw
			opengl_draw
			render_state := rs_postdraw
			postdraw
			render_state := rs_unknown
			if frame_counter/=void then
				-- Allow opengl blitting
				screen.enable_auto_opengl_blitting
				frame_counter.draw ( screen )
			end
			screen.redraw
		end
	update_light is
			-- Update local global light position
		do
			emgl_load_identity
			if active_camera/=void then
				active_camera.set_opengl
			end
			emgl_enable( em_gl_lighting )
			scene_manager.light.render_opengl
		end

	opengl_draw is
			-- Do the actual drawing. Dont overwrite this function
			-- unless you want to flush the buffers on you own
		do
			-- Reset depth buffer and clear the screen
			emgl_clear (Em_gl_color_buffer_bit | Em_gl_depth_buffer_bit)
			-- Clear modelview matrix
			emgl_load_identity

			if active_camera/=void then
				-- We want auto_opengl_bliting
				screen.enable_auto_opengl_blitting

				active_camera.set_opengl
			end
--			scene_manager.update_world
			-- Draw scene
			draw
		end

	predraw is
			-- This procedure is called before drawing, you might use it for
			-- RenderToTexture, ...
		do
		end

	draw is
			-- Draw scene.
		do
			scene_manager.render_world
		end

	postdraw is
			-- This procedure is called after the drawing, if you want to change
			-- something after the drawing, or clean up anything, do it here, but
			-- don't clear the color_buffer!
		do
		end

end
