indexing
	description: "[
		
		Cached glyph that can draw itself using OpenGL, used in EM3D_TTF_FONT

	]"
	author: ""
	date: "$Date: 2007-01-15 22:43:41 +0100 (Mon, 15 Jan 2007) $"
	revision: "$Revision: 2403 $"

class EM3D_TTF_GLYPH

inherit

	EM_CONSTANTS
		export {NONE} all end

	EMGL_SETTINGS
		export {NONE} all end

	EMGL_BASIC_RENDERER
		export {NONE} all end

	EMGL_VIEW
		export {NONE} all end

create {EM3D_TTF_FONT}

	make

feature {NONE} -- Initialization

	make (unicode: INTEGER; a_ttf_font: EM_TTF_FONT) is
			-- Create new glyph from `a_ttf_font', representing `unicode' character
		local
			surf: EM_SURFACE
		do
			metrics := a_ttf_font.glyph_metrics (unicode)

			-- Color does not matter, we're interested in alpha only.
			-- But white seems like a good choice anyway.
			surf := a_ttf_font.create_glyph_surface (unicode, create {EM_COLOR}.make_white)

			-- Disable blending, otherwise SDL won't copy alpha channel to
			-- texture!
			surf.disable_alpha_transparency

			-- We add a border of 1 around every glyph to prevent artifacts
			create texture.make_mipmap_from_surface_with_border (surf, 1)
			texture.set_texture_mag_filter (em_gl_linear)
			texture.set_texture_min_filter (em_gl_linear_mipmap_linear)
			-- Account for border
			tex_coords.set_x (+0.0 / texture.texture_width)
			tex_coords.set_y (+0.0 / texture.texture_height)
			tex_coords.set_z ((surf.width + 2.0) / texture.texture_width)
			tex_coords.set_w ((surf.height + 2.0) / texture.texture_height)
		ensure
		end

feature -- Access

	metrics: EM_TTF_GLYPH_METRICS
		-- Metrics (in pixels) of glyph

	texture: EMGLU_TEXTURE_2D[EMGL_FORMAT_RGBA]
		-- Glyph bitmap as OpenGL texture

feature -- Drawing

	render is
			-- Draw current glyph as textured quad in OpenGL
			-- Set color and modelview matrix accordingly before
		local
			x1, y1, x2, y2: INTEGER
		do
			x1 := metrics.min.x
			x2 := metrics.max.x
			y1 := metrics.min.y
			y2 := metrics.max.y

			if false then -- draw outlines when debugging
				emgl_disable (em_gl_texture_2d)
				emgl_begin (em_gl_line_loop)
				emgl_vertex2i (x1-1, -y2-1)
				emgl_vertex2i (x2+1, -y2-1)
				emgl_vertex2i (x2+1, -y1+1)
				emgl_vertex2i (x1-1, -y1+1)
				emgl_end
				emgl_enable (em_gl_texture_2d)
			end

			-- The -/+1 offset is because of the extra texture borders
			texture.bind
			emgl_begin (em_gl_quads)
			emgl_tex_coord2d (tex_coords.x, tex_coords.y)
			emgl_vertex2i (x1-1, -y2-1)
			emgl_tex_coord2d (tex_coords.z, tex_coords.y)
			emgl_vertex2i (x2+1, -y2-1)
			emgl_tex_coord2d (tex_coords.z, tex_coords.w)
			emgl_vertex2i (x2+1, -y1+1)
			emgl_tex_coord2d (tex_coords.x, tex_coords.w)
			emgl_vertex2i (x1-1, -y1+1)
			emgl_end

			emgl_translated (metrics.advance, 0.0, 0.0)
		end

feature {NONE} -- Implementation

	tex_coords: EM_VECTOR4D
		-- Fractional texture coords

end
