indexing

	description: "[

		Represents a transformation and holds a list of moveable objects (for example Entities, Lights, Cameras, ...).
		It also enhances the `NODE' class by several convenience functions like `lookAt' or `setAutoTrackTarget'.
		NOTE: The current implementation is only a small subset of what a scene node should support in the end. For
		example scene notes should also support features to get bounding boxes and stuf like that which can be used for
		collision detection, for building datastructres which perform culling and so on.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EM3D_SCENE_NODE

inherit

	EM3D_NODE
	redefine make_by_parent end

	EMGL_VIEW
	export {NONE} all
	undefine copy, is_equal end

	EM3D_VISIBILITY_SUPPORT
	undefine copy, is_equal end


create

	make_by_parent

feature -- Initialization
	make_by_parent ( a_parent: like Current ) is
			-- Create a new node with a given parent
		do
			Precursor (a_parent)
			create {DS_LINKED_LIST[EM3D_MOVABLE]} objects.make_default
			set_visible (true)
		ensure then
			is_visible: is_visible
		end

feature -- Access

	object_by_name (a_name: STRING): EM3D_MOVABLE is
			-- Returns a reference to an object with name `a_name'
		require
			object_in_list: has_object_by_name (a_name)
		do
			check false end
		end

feature -- Element change
	new_child: like Current is
			-- Returns a new child scene node.
		do
			create Result.make_by_parent (Current)
		end

	attach_object (an_object: EM3D_MOVABLE) is
			-- Adds a movable object to the current node.
		require
			an_object_not_void: an_object /= Void
			not_already_added: not has_object (an_object) -- one may argue about this. clearly: each object should only be once in the list
		do
			objects.put_last (an_object)
			an_object.set_parent (Current)
		end

feature -- Status setting

	enable_orientation_inheritance is
			-- Enables the inheritance of parents orientation.
		local
			a: ANY
		do
			is_inheriting_orientation_from_parent := True
		ensure
			status_set: is_inheriting_orientation_from_parent
		end

	enable_scale_inheritance is
			-- Enables the inheritance of parents scale.
		do
			is_inheriting_scale_from_parent := True
		ensure
			status_set: is_inheriting_scale_from_parent
		end

	disable_orientation_inheritance is
			-- Enables the inheritance of parents orientation.
		do
			is_inheriting_orientation_from_parent := False
		ensure
			status_set: not is_inheriting_orientation_from_parent
		end

	disable_scale_inheritance is
			-- Enables the inheritance of parents scale.
		do
			is_inheriting_scale_from_parent := False
		ensure
			status_set: not is_inheriting_scale_from_parent
		end



feature -- Status report

	has_object (an_object: EM3D_MOVABLE): BOOLEAN is
			-- Is `an_object' attached to this node?
		do
			Result := objects.has (an_object)
		end

	has_object_by_name (a_name: STRING): BOOLEAN is
			-- Is `an_object' attached to this node?
		do
			check false end -- TODO: implement
			-- either we have a global name space, or not.
			-- depending on this decision this feature needs a different implementation
		end

	is_inheriting_orientation_from_parent: BOOLEAN
			-- Is orientation inherited from parent node?

	is_inheriting_scale_from_parent: BOOLEAN
			-- Is scale inherited from parent node?

feature -- Removal

	remove_object (an_object: EM3D_MOVABLE) is
			-- Remove `an_object'.
		do
			objects.delete (an_object)
			an_object.set_parent (Void)
		ensure
			object_removed: not  has_object (an_object)
		end

	remove_object_by_name (a_name: STRING): BOOLEAN is
			-- Remove the object with name `a_name'.
		require
			object_in_list: has_object_by_name (a_name)
		do
			remove_object (object_by_name (a_name))
		ensure
			object_removed: not  has_object_by_name (a_name)
		end

feature {EM3D_SCENE_NODE, EM3D_SCENE_MANAGER} -- Basic operation

	render_opengl is
			-- TODO: i just did this for testing... remove or change to new render pipeline..
		local
			child: DS_LIST_CURSOR[like Current]
			object: DS_LIST_CURSOR[EM3D_MOVABLE]
			r: EM_VECTOR3D
		do
				-- first setup opengl transformation
				emgl_push_matrix

				emgl_mult_matrixd ( transformation )

				emgl_scale ( scale )

				-- TODO: apply all stuff which is inherited from "render_parameters"

				-- Render all attached objects
				object := objects.new_cursor
				from object.start
				until
					object.after
				loop
					if object.item.is_visible then
						object.item.render_opengl
					end
					object.forth
				end

				-- Visit all children and render them too
				child := children.new_cursor
				from child.start
				until
					child.after
				loop
					if child.item.is_visible then
						child.item.render_opengl
					end
					child.forth
				end

				emgl_pop_matrix

		end


feature {NONE} -- Implementation

	objects: DS_LIST[EM3D_MOVABLE]

end -- class EM3D_SCENE_NODE
