indexing
	description: "[
			A very simple implementation of a light.
			It does not reuse OpenGL lights and due to this
			the number lights is limited to 8
		]"
	date: "$Date: Jul 23, 2006 3:34:30 PM$"
	revision: "$Revision: 1.0$"

class EM3D_LIGHT
inherit
	EMGL_LIGHT
		rename
			make as make_opengl,
			is_enabled as is_visible
		undefine
			copy,
			is_equal,
			is_visible
		redefine
			enable, disable
		end
	EMGL_VIEW
		export
			{NONE} all
		undefine
			copy,
			is_equal
		end

	EM3D_MOVABLE
		rename
			render_opengl as do_nothing
		redefine
			default_name,
			type,
			make,
			set_visible
		end
create {EM3D_OBJECT_MANAGER}
	make
feature {EM3D_IDENTIFIED} -- Identified
	default_name: STRING is
			-- Generate the default name from the class name
		once
			result := "LIGHT"
		end
	type: INTEGER is
			-- Generate the default name from the class name
		once
			result := em3d_light_identified
		end

feature {NONE} -- Initialization
	make(a_scene_manager: EM3D_SCENE_MANAGER) is
			-- Create a new EM3D_LIGHT object.
		do
			Precursor(a_scene_manager)
			if a_scene_manager/=void then
				-- Set the default values
				set_ambient( [0.1,0.1,0.1,1.0] )
				set_diffuse( [1,1,1,1] )
				set_specular( [0.2,0.2,0.2,0.0] )
				set_exponent( 0 )
				set_cutoff( 180 )
				set_constant_attenuation ( 1 )
				set_linear_attenuation ( 0 )
				set_quadratic_attenuation ( 0 )
				enable
			end
		end

feature -- Visibility support
	enable is
			-- Enable the light
		require else
			true
		do
			is_visible := true
		end
	disable is
			-- Enable the light
		require else
			true
		do
			is_visible := false
		end
	set_visible( a_value: BOOLEAN ) is
			-- Specify if the light is turned on or off
		do
			if a_value then
				enable
			else
				disable
			end
		end

feature {EM3D_LIGHT_MANAGER} -- Rendering
	render_opengl is
		local
			light_id : INTEGER
			tmp: EM_VECTOR3F
			pos: EM_VECTOR4F
		do
			if is_visible then
				light_id := scene_manager.light.new_id
				if is_valid_light( light_id ) then
					make_opengl( light_id )

					emgl_enable( light_id )

					pos := parent.world_position
					pos.set_w( 1 )
					lightfv( em_gl_position, pos )
					if target=void then
						tmp := [0,0,-1]
					else
						tmp := target.position - world_position
					end
					lightfv( em_gl_spot_direction, tmp )

					lightfv( em_gl_ambient, ambient )
					lightfv( em_gl_diffuse, diffuse )
					lightfv( em_gl_specular, specular )
					lightf( em_gl_spot_exponent, exponent )
					lightf( em_gl_spot_cutoff, cutoff )
					lightf( em_gl_constant_attenuation, constant_attenuation )
					lightf( em_gl_linear_attenuation, linear_attenuation )
					lightf( em_gl_quadratic_attenuation, quadratic_attenuation )
				end
			end
		end

feature -- Status
	ambient: EM_VECTOR4F
	set_ambient( a_value: EM_VECTOR4F ) is
			-- Specify the ambient light
		do
			ambient := a_value
		end

	diffuse: EM_VECTOR4F
	set_diffuse( a_value: EM_VECTOR4F ) is
			-- Specify the diffuse light
		do
			diffuse := a_value
		end

	specular: EM_VECTOR4F
	set_specular( a_value: EM_VECTOR4F ) is
			-- Specify the specular light
		do
			specular := a_value
		end

	target: EM3D_TARGET
	set_target( a_value: like target ) is
			-- Specify the light direction as a target
		do
			target := a_value
		end

	exponent: REAL
	set_exponent( a_value: REAL ) is
			-- Specify the intensity distribution of the light
		do
			exponent := a_value
		end

	cutoff: REAL
	set_cutoff( a_value: REAL ) is
			-- Specify the maximum spread angle of the light source
		require
			valid_cutoff: a_value >= 0 and (a_value<=90 or a_value=180)
		do
			cutoff := a_value
		end

	constant_attenuation: REAL
	set_constant_attenuation( a_value: REAL ) is
			-- Specify the constant attenuation
		do
			constant_attenuation := a_value
		end

	linear_attenuation: REAL
	set_linear_attenuation( a_value: REAL ) is
			-- Specify the linear attenuation
		do
			linear_attenuation := a_value
		end

	quadratic_attenuation: REAL
	set_quadratic_attenuation( a_value: REAL ) is
			-- Specify the quadratic attenuation
		do
			quadratic_attenuation := a_value
		end

end -- class EM3D_LIGHT
