indexing
	description: "A simple 3D arrow"
	date: "$Date$"
	revision: "$Revision$"

class
	EM3D_ARROW

inherit
	EM3D_SPECIAL_OBJECT
		rename
			make as make_movable
		end

	EM_GLU_CONSTANTS
		export {NONE} all
		undefine copy, is_equal end
	EMGL_VIEW
		export {NONE} all
		undefine copy, is_equal end

create
	make, make_default

feature {NONE} -- Initialization
	make ( a_source, a_destination: EM_VECTOR3D ) is
			-- Create a new EM3D_ARROW pointing from 'a_source' to 'a_destination'
		do
			source := a_source
			destination := a_destination
			radius := 0.15*length
			head_size := 0.25*length
			head_radius := 0.25*length
			draw_style := em_glu_fill
		end
	make_default is
			-- Create a new EM3D_ARROW pointing from (0,0,0) to (0,0,1)
		do
			make( [0,0,0], [0,0,1] )
		end


feature -- Status
	radius, head_radius: DOUBLE
	head_size: DOUBLE
	draw_style: INTEGER
	source, destination: EM_VECTOR3D
	length:DOUBLE is
			-- The length of the arrow
		do
			result := (source-destination).length
		end

feature -- Status setting
	set_radius( a_radius: like radius ) is
			-- Specify the 'radius'
		do
			radius := a_radius
		ensure
			radius_set: radius = a_radius
		end
	set_head_radius( a_head_radius: like head_radius ) is
			-- Specify the 'head_radius'
		do
			head_radius := a_head_radius
		ensure
			head_radius_set: head_radius = a_head_radius
		end
	set_head_size( a_head_size: like head_size ) is
			-- Specify the 'head_size'
		require
			valid_head_size: 0 < a_head_size and a_head_size < 1
		do
			head_size := a_head_size
		ensure
			head_size_set: head_size = a_head_size
		end
	set_draw_style( a_draw_style: like draw_style ) is
			-- Specify the 'draw_style'
		do
			draw_style := a_draw_style
		ensure
			draw_style_set: draw_style = a_draw_style
		end
	set_source( a_source: like source ) is
			-- Specify the 'source'
		do
			source := a_source
		ensure
			source_set: source = a_source
		end
	set_destination( a_destination: like destination ) is
			-- Specify the 'sodestination'
		do
			destination := a_destination
		ensure
			destination_set: destination = a_destination
		end

feature {NONE} -- Implementation
	qr: EMGLU_QUADRIC is
			-- The quadric renderer used to draw the Arrow elements
		once
			create result
			result.normals (em_glu_smooth)
			result.draw_style (em_glu_fill)
		end

feature -- Drawing
	render_opengl is
			-- Draw the arrow
		local
			q: EM_QUATERNION
			a: EM_VECTOR3D
		do
			emgl_push_matrix
			emgl_translatef( source.x, source.y, source.z )
			q.set_from_arc ( [0,0,1], destination-source )
			a := q.axis_of_rotation
			emgl_rotatef( q.angle_of_rotation_degree, a.x, a.y, a.z )
			qr.cylinder( radius, radius, length-head_size, 20, 1 )

			qr.orientation( em_glu_inside )
			emgl_push_matrix
			qr.disk( 0, radius, 20, 1 )
			emgl_translatef( 0, 0, length-head_size )
			qr.disk( 0, head_radius, 20, 1 )

			qr.orientation( em_glu_outside )
			qr.cylinder( head_radius, 0, head_size, 20, 1 )
			emgl_pop_matrix
			emgl_pop_matrix
		end

invariant
	valid_head_size: 0 < head_size and head_size < length

end
