indexing
	description: "A selection renderer object"
	date: "$Date: Aug 27, 2006 11:31:37 AM$"
	revision: "$Revision: 1.0$"

deferred class EM3D_SELECTION_OBJECT

inherit
	EWG_IDENTIFIED
		rename
			ewg_object_id as id
		export
			{EM3D_SELECTION_OBJECT} all
			{ANY} id
		end

	EMGL_SELECT
		export {NONE} all
		undefine is_equal, copy end


feature -- Access
	frozen has_object( an_id: INTEGER ): BOOLEAN is
			-- Does 'an_id' exist?
		require
			an_id_is_positive: id>0
		local
			tmp: like current
		do
			tmp ?= ewg_id_object( an_id )
			result:= tmp/=void
		end
	frozen find_object( an_id: INTEGER ): EM3D_SELECTION_OBJECT is
			-- Find a selection object matching 'an_id'
		require
			an_id_is_positive: id>0
		do
			result ?= ewg_id_object( an_id )
		end

feature --Commands
	push_name is
			-- Push the objects name into the namestack
			-- Needs to be called before rendering
			-- Avoid a too deep namestack, because it might
			-- easily overload the selection buffer. Try not
			-- to call more than 5 more pushes than pop's
		do
			emgl_push_name( id )
		end
	pop_name is
			-- Pop the objects name out of the namestack
			-- Needs to be called after rendering
		do
			emgl_pop_name
		end
	load_name is
			-- Load the objects name into the namestack
			-- Alternative to push/pop_name, the namestack
			-- is not overloaded with many objects
		do
			emgl_load_name( id )
		end
	load_default_name is
			-- Load the objects name into the namestack
			-- Alternative to push/pop_name, the namestack
			-- is not overloaded with many objects
		do
			emgl_load_name( -1 )
		end

feature {EM3D_SELECTION_RENDERER} --Callback
	selected ( z_near, z_far: DOUBLE ) is
			-- This function is called as soon as the object is selected
			-- z_near: The minimum z value among the primitives hit
			-- z_far:  The maximum z value among the primitives hit
		deferred
		end

invariant
	valid_id: id>0

end -- class EM3D_SELECTION_OBJECT
