indexing
	description: "[
			OpenGL provides several rendermodes (Drawing, selectionmode, feedbackmode,...).
			In Eiffel we try to handle this modes with a number of 'renderers'. These 'renderers'
			are interfaces to the basic Opengl commands for those rendermodes.
			
			The selection renderer can use selection to determine which primitives are  drawn
			into  some  region  of  a window.  The region is defined by the current
			modelview and perspective matrices. 
			
			You can name the primitives by calling glLoadName or by using the EM3D_SELECTION_OBJECT.
		]"
	date: "$Date: Aug 27, 2006 11:06:48 AM$"
	revision: "$Revision: 1.0$"

class EM3D_SELECTION_RENDERER

inherit
	ANY

	EM3D_SHARED_SELECTION_OBJECT_FINDER
		export {NONE} all end

	EMGL_SELECT
		export {NONE} all end

	EMGL_VIEW
		export {NONE} all end

	EMGLU_VIEW
		export {NONE} all end

	EMGL_SETTINGS
		export {NONE} all end

	EM_CONSTANTS
		export {NONE} all end

	DISPOSABLE
		export {NONE} all end

create
	make

feature {NONE} -- Initialization

	make( a_buffer_size: INTEGER ) is
			-- Create a new EM3D_SELECTION_RENDERER object.
		do
			create buffer.make_new_unshared( a_buffer_size )
		ensure
			buffer_created: buffer/=void
		end

	dispose is
			-- Unbind the buffer before deleting it
		do
			-- TODO: Add some protection that the memory is not deleted
			--       when it's still used in select mode
		end

feature {EM3D_SELECTION_RENDERER} -- Access
	buffer: EWG_MANAGED_POINTER
	maxint: NATURAL_32 is 0xffffffff

feature -- Command
	begin_picking_select(a_position, a_delta: EM_VECTOR2D) is
			-- Start selection with picking a small area `a_delta'
			-- around `a_position'
		local
			viewport: EM_VECTOR4I
			proj_mat: EM_MATRIX44
		do
			begin_select
			proj_mat := emgl_get_projection_matrix
			viewport := emgl_get_viewport

			emgl_matrix_mode( em_gl_projection )
			emgl_load_identity
			-- In OpenGL Coordinates the y is inverted
			emglu_pick_matrix( a_position.x, viewport.y+viewport.w-a_position.y, a_delta.x, a_delta.y, viewport)
			emgl_mult_matrixd ( proj_mat )
			emgl_matrix_mode( em_gl_modelview )
		end


	begin_area_select(a_p1, a_p2: EM_VECTOR2D) is
			-- Start selection with picking a small area `a_delta'
			-- around `a_position'
		local
			proj_mat: EM_MATRIX44
			viewport: EM_VECTOR4I
			sx, sy: REAL
			mx, my: REAL
		do
			begin_select
			proj_mat := emgl_get_projection_matrix
			viewport := emgl_get_viewport
			mx := (a_p2.x+a_p1.x) / (viewport.z-viewport.x)
			my := (a_p2.y+a_p1.y) / (viewport.w-viewport.y)
			if a_p1.x = a_p2.x then
				sx := (viewport.z-viewport.x)
			else
				sx := (viewport.z-viewport.x) / (a_p2.x-a_p1.x)
			end
			if a_p1.y = a_p2.y then
				sy := (viewport.w-viewport.y)
			else
				sy := (viewport.w-viewport.y) / (a_p2.y-a_p1.y)
			end
			emgl_matrix_mode( em_gl_projection )
			emgl_load_identity
			emgl_scalef( sx, sy, 1)
			emgl_translatef( 1-mx, my-1, 0)
			emgl_mult_matrixd ( proj_mat )
			emgl_matrix_mode( em_gl_modelview )
		end


	begin_select is
			-- Start the select mode rendering
			-- Select all objects that are visible on the screen
		local
			tmp: INTEGER
		do
			emgl_select_buffer( buffer.capacity, buffer.item )
			emgl_init_names
			emgl_push_name(-1)
			tmp := emgl_render_mode( em_gl_select )
			emgl_matrix_mode( em_gl_projection )
			emgl_push_matrix
			emgl_matrix_mode( em_gl_modelview)
		end
	end_select is
			-- Stop the select mode rendering
		local
			n_hits, stack_depth, id: INTEGER
			i, j, pos: INTEGER
			z_near, z_far: NATURAL_32
			object: EM3D_SELECTION_OBJECT
		do
			emgl_matrix_mode( em_gl_projection )
			emgl_pop_matrix
			emgl_matrix_mode( em_gl_modelview)

			n_hits := emgl_render_mode( em_gl_render )
			pos := 0
			from i:=1 until i>n_hits loop
				stack_depth := buffer.read_integer( pos ) ; pos := pos + 4
				z_near := buffer.read_integer( pos ).as_natural_32 ; pos := pos + 4
				z_far := buffer.read_integer( pos ).as_natural_32 ; pos := pos + 4
				from j:=1 until j>stack_depth loop
					id := buffer.read_integer( pos ) ; pos := pos + 4
					object := selection_object_finder.find_object( id )
					if object /= void then
						object.selected( z_near/maxint, z_far/maxint )
					end
					j := j + 1
				end
				i := i + 1
			end
		end

end -- class EM3D_SELECTION_RENDERER
