#include "trie.h"
#include <stdlib.h>
#include <ewg_eiffel.h>

#define BASIC_SIZE 16

#define INIT_VAR( var ) if ( !(var) ) (var) = malloc( BASIC_SIZE * sizeof( *(var) ) );
#define RESIZE_VAR( var, size ) (var) = realloc( (var), (size) * sizeof( *(var) ) );


#ifdef __cplusplus
extern "C" {
#endif

Trie ** names = NULL;
int max_type_id = 0;
int type_capacity = BASIC_SIZE;

// Types
int em3d_new_type( void ){
	INIT_VAR( names );
	
	if (max_type_id > type_capacity){
		type_capacity *= 2;
		RESIZE_VAR( names, type_capacity );
	}
	names[ max_type_id ] = newTrie();
	return max_type_id++;
}

int em3d_is_valid_type( int type ){
	return 0 <= type && type < max_type_id;
}


// Object names
int em3d_name_object( int type, char * name ){
	if ( type >= max_type_id ) return 0;
	return (int)TrieSearch( names[ type ], name );
}
void em3d_object_remove_name( int type, char * name ){
	if ( type < max_type_id )
		TrieRemove( names[ type ], name );
}

void em3d_object_set_name( int type, char * name, int object ){
	if ( type < max_type_id )
		TrieInsert( names[ type ], name, (TRIE_TYPE)object );
}

/*
void em3d_foreach_object( int type, TrieCallback c){
	if ( type < max_type_id )
		TrieForeach( names[ type ], c );
}*/

#ifdef __cplusplus
}
#endif
