/*
trie.h

Copyright (C) 2006  Philipp Kraehenbuehl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/
#ifndef _TRIE_H_
#define _TRIE_H_

#ifndef TRIE_TYPE
#define TRIE_TYPE int
#endif
struct TNode {
	struct TNode * next;
	struct TNode * down;
	char * key;
	TRIE_TYPE element;
};
typedef struct TNode TrieNode;

struct T{
	TrieNode * root;
};
typedef struct T Trie;

typedef void (* TrieCallback)( TRIE_TYPE index );

Trie* newTrie( void );
void freeTrie( Trie* t );

// Memory for n Elements O(n) , <=16n + sizeof(word)

// O( strlen( key ) )
TrieNode * TrieInsert( Trie *t, char* key, TRIE_TYPE element );
// O( strlen( key ) )
void TrieRemove( Trie *t, char* key );
// O( strlen( key ) )
TRIE_TYPE TrieSearch( Trie *t, char* key );
// O( sizeof( tree ) )
void TrieForeach( Trie * t, TrieCallback c );
#endif
