indexing

	description: "[

		Event that notifies peer clients to destroy a specific object.
		The object with ID `destroy_id' will be destroyed.
		
		The object will (by default) be destroyed in the same group
		as the event is sent from.
		
		After the success agent was called, the event caller can assume that the object
		has been destroyed on all remote peers that belong to the sending group.
		
		This is a secured two phase commit event.

	]"
	date: "$Date: 2005-10-26 16:05:23 +0200 (Wed, 26 Oct 2005) $"
	revision: "$Revision: 1295 $"

class EM_NET_DESTROY_OBJECT_RESPONSE

inherit

	EM_NET_2PC_EVENT_OBJECT

create {EM_NET_OBJECT_TYPES}

	make_set_type

feature -- Element change

	set_destroy_id (an_id: INTEGER) is
			-- Set `destroy_id' to `an_id'.
		do
			destroy_id := an_id
		end

feature -- Information

	destroy_id: INTEGER
			-- Unique object ID of the object to be destroyed

feature -- Serialization

	serialize (a_serializer: EM_NET_SERIALIZER) is
			-- Serialize to the given stream using `a_serializer'.
		do
			a_serializer.put_integer (id)
			a_serializer.put_integer (destroy_id)
		end

	serialization_byte_count: INTEGER is
			-- Count of bytes needed for a successful serialization
		do
			Result := 8
		end

	unserialize (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Unserialize from a given stream using `an_unserializer'.
		do
			an_unserializer.read_integer
			id := an_unserializer.last_integer

			an_unserializer.read_integer
			destroy_id := an_unserializer.last_integer
		end

end