indexing

	description: "[

		Event that contains status information of a peer.
		
		This might be sent as a response to a EM_NET_STATUS_REQUEST.

	]"
	date: "$Date: 2005-10-23 14:18:39 +0200 (Sun, 23 Oct 2005) $"
	revision: "$Revision: 1148 $"

class EM_NET_STATUS_MESSAGE

inherit

	EM_NET_EVENT_OBJECT
		redefine
			make_set_type
		end

create {EM_NET_OBJECT_TYPES}

	make_set_type

feature {NONE} -- Initialisation

	make_set_type (a_type_id: INTEGER) is
			-- Set `create_type_id' to `a_type_id'.
		do
			Precursor {EM_NET_EVENT_OBJECT} (a_type_id)
			item := ""
		end

feature -- Access

	item: STRING
			-- Status string

feature -- Serialization

	serialize (a_serializer: EM_NET_SERIALIZER) is
			-- Serialize to the given stream using `a_serializer'.
		do
			a_serializer.put_integer(item.count)
			a_serializer.put_string(item)
		end

	serialization_byte_count: INTEGER is
			-- Count of bytes needed for a successful serialization
		do
			Result := 4 + item.count
		end

	unserialize (an_unserializer: EM_NET_UNSERIALIZER) is
			-- Unserialize from a given stream using `an_unserializer'.
		local
			count: INTEGER
		do
			an_unserializer.read_integer
			count := an_unserializer.last_integer
			if count = 0 then
				item := ""
			else
				an_unserializer.read_string(count)
				item := an_unserializer.last_string
			end
		end

feature -- Element change

	set_status(a_status_string: STRING) is
			-- Set `item' to `a_status_string'.
		require
			a_status_string_not_void: a_status_string /= Void
		do
			item := a_status_string
		ensure
			status_string_set: item = a_status_string
		end

feature {NONE} -- Implementation

end