indexing

	description: "[

		Base type for event objects.
		
		Network event objects are different from normal network objects in such a way
		that they only have a short lifetime. After being published to remote peers, the
		network event has no more reason to live any longer and will be discarded.
		As a consequence, network event can not be synchronized between peers.
		
		Also a network event doesn't necessarily have to have an unique ID, but you may
		use the ID for you own purpose. In this case you might want to serialize/unserialize
		the ID manually.

	]"
	date: "$Date: 2005-10-26 16:05:23 +0200 (Wed, 26 Oct 2005) $"
	revision: "$Revision: 1295 $"

deferred class	EM_NET_EVENT_OBJECT

inherit

	EM_NET_OBJECT
		redefine
			set_group, is_synchronisation_needed
		end

feature -- Event handling

	publish is
			-- Publish an event to the clients that belong to `group'.
		require
			group_set: group /= Void
		do
			group.net_event_container_object.publish (Current)
		end

feature -- Group handling.

	set_group (a_group: EM_NET_GROUP) is
			-- Set `group' to `a_group'.
			-- This is the group to which this event will be published.
		do
			group := a_group
		end

feature {NONE} -- Implementation

	is_synchronisation_needed : BOOLEAN is
			-- Is synchronisation needed?
			-- False by default.
		do
			Result := True
		end

end