indexing

	description: "[

		Provide mapping of error constants from EM_ERROR_CODES to string messages.
		
		Placeholders are marked with '{#}' where '#' is the number of replacement.
		Zero is always the error code. 1, 2, etc. are indexes in the error data tuple.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EM_ERROR_MESSAGES

inherit

	EM_ERROR_CODES
		export {NONE} all end

feature {NONE} -- Initialisation

	make_default_messages is
			-- Initialise default error messages.
		do
			-- Setup default error message
			Error_messages.force ("Errorcode '{0}' - See em.utility.EM_ERROR_CODES for more information", Em_error_default)

			-- Video system error messages
			Error_messages.force ("Requested video mode not supported. (width={1} / height={2} / bpp={3} / flags={4})", Em_error_video_mode_not_supported)
			Error_messages.force ("Could not set the video mode: {1}.", Em_error_set_video_mode)
			Error_messages.force ("Failed to initialise screen", Em_error_initializing_screen)
			Error_messages.force ("Failed to toggle fullscreen mode", Em_error_toggle_fullscreen)
			Error_messages.force ("Unable to toggle fullscreen on current operating system (only X11 Display server is supported)", Em_error_toggle_fullscreen_not_supported)
			Error_messages.force ("Failed to create cursor", Em_error_create_cursor)

			-- Surface error messages
			Error_messages.force ("Failed to lock surface", Em_error_locking_surface)
			Error_messages.force ("Failed to set alpha value of surface to '{1}'", Em_error_set_alpha_value)
			Error_messages.force ("Failed to enable alpha transparency of surface", Em_error_enable_alpha_transparency)
			Error_messages.force ("Failed to disable alpha transparency of surface", Em_error_disable_alpha_transparency)
			Error_messages.force ("Failed to enable per pixel alpha of surface", Em_error_enable_per_pixel_alpha)
			Error_messages.force ("Failed to set transparent colorkey of surface to '{1}'", Em_error_set_transparent_colorkey)
			Error_messages.force ("Failed to disable transparent colorkey of surface", Em_error_disable_transparent_colorkey)
			Error_messages.force ("Failed to apply rotozoom on surface with x factor '{1}', y factor '{2}' and rotation angle '{3}'", Em_error_apply_rotozoom)
			Error_messages.force ("Failed to blit surface", Em_error_blit_surface)
			Error_messages.force ("Failed to blit surface part", Em_error_blit_surface_part)
			Error_messages.force ("Failed to save surface to file (%"{1}%")", Em_error_save_surface)

			-- Bitmap factory error messages
			Error_messages.force ("Failed to load image file '{1}'. File format not supported", Em_error_loading_image_format_not_supported)
			Error_messages.force ("Failed to load image file '{1}'. File doesn't exist", Em_error_loading_image_does_not_exist)

			Error_messages.force ("Failed to create empty bitmap of dimension ({1}/{2})", Em_error_create_empty_bitmap)
			Error_messages.force ("Failed to create empty bitmap of dimension ({1}/{2}) with alpha transparency", Em_error_create_empty_alpha_bitmap)
			Error_messages.force ("Failed to create empty OpenGL bitmap of dimension ({1}/{2})", Em_error_create_empty_opengl_bitmap)
			Error_messages.force ("Failed to create bitmap from surface part - surface position ({1}/{2}) - part dimension ({3}/{4})", Em_error_create_bitmap_from_surface_part)
			Error_messages.force ("Failed to create zoomed bitmap with zoom factor '{1}'", Em_error_create_zoomed_bitmap)
			Error_messages.force ("Failed to create stretched bitmap with x factor '{1}' and y factor '{2}'", Em_error_create_stretched_bitmap)
			Error_messages.force ("Failed to create rotated bitmap with angle '{1}'", Em_error_create_rotated_bitmap)
			Error_messages.force ("Failed to create transformed bitmap with x factor '{1}', y factor '{2}' and rotation angle '{3}'", Em_error_create_transformed_bitmap)
			Error_messages.force ("Failed to create empty surface of dimension ({1}/{2})", Em_error_create_empty_surface)
			Error_messages.force ("Failed to create empty surface of dimension ({1}/{2}) with alpha transparency", Em_error_create_empty_alpha_surface)

			-- Texture error messages
			Error_messages.force ("Failed to create mipmap texture. OpenGL error message is '{1}'", Em_error_generate_mipmap)
			Error_messages.force ("Failed to create texture. OpenGL error message is '{1}'", Em_error_generate_texture)

			-- Font error messages
			Error_messages.force ("Failed to initialise TTF system", Em_error_initialising_ttf)
			Error_messages.force ("Failed to load TTF font file '{1}'", Em_error_loading_ttf_font)
			Error_messages.force ("Failed to load bitmap font file '{1}'", Em_error_load_bitmapfont)

			-- Keyboard error messages
			Error_messages.force ("Failed to enable unicode characters of keyboard events", Em_error_enable_unicode)
			Error_messages.force ("Failed to disable unicode characters of keyboard events", Em_error_disable_unicode)
			Error_messages.force ("Failed to enable repeating keyboard events (delay: {1} / interval: {2})", Em_error_enable_repeating_keyboard_events)
			Error_messages.force ("Failed to disable repeating keyboard events", Em_error_disable_repeating_keyboard_events)

			-- Widget error messages
			Error_messages.force ("Unable to show combobox list selection because running scene is void", Em_error_show_combobox_list_scene_void)
			Error_messages.force ("Unable to show combobox list selection because running scene is no EM_COMPONENT_SCENE", Em_error_show_combobox_list_scene_invalid)
			Error_messages.force ("Unable to hide combobox list selection because running scene is void", Em_error_hide_combobox_list_scene_void)
			Error_messages.force ("Unable to hide combobox list selection because running scene is no EM_COMPONENT_SCENE", Em_error_hide_combobox_list_scene_invalid)
			Error_messages.force ("Unable to show window because running scene is void", Em_error_show_window_scene_void)
			Error_messages.force ("Unable to show window because running scene is no EM_COMPONENT_SCENE", Em_error_show_window_scene_invalid)

			-- Video Decoder error messages
			Error_messages.force ("Failed to load movie file '{1}'. File is no movie file or codec isn't supported", Em_error_create_video_decoder_from_file)
			Error_messages.force ("Failed to load movie file '{1}'. File is not a valid {2} movie file", Em_error_create_video_decoder_from_file_not_valid)
			Error_messages.force ("Failed to load movie file '{1}'. File doesn't exist", Em_error_create_video_decoder_from_file_does_not_exist)

			-- Miscellaneous error messages
			Error_messages.force ("Unknown event of type '{1}' occured in event loop", Em_error_unknown_event_type)
			Error_messages.force ("A user event of type '{1}' occured which is a network event. But network is disabled", Em_error_user_event_code_used_by_network)
			Error_messages.force ("A 'this should never happen' error has occured in section '{1}'. Please notify the developers", Em_error_this_should_never_happen)

			-- Filehandler
			Error_messages.force ("Failed to load text file '{1}'. File doesn't exist",Em_error_plain_text_file_does_not_exist)
			Error_messages.force ("Failed to read from file '{1}'. File not readable",Em_error_plain_text_file_not_readable)

			-- Network
			Error_messages.force ("Net2 error occured: '{1}'",Em_error_net2_error)
			Error_messages.force ("Failed to open tcp server socket on port '{1}'. Port is probably taken", Em_error_tcp_open)
			Error_messages.force ("Failed to open udp socket on port '{1}'. Port is probably taken", Em_error_udp_open)
			Error_messages.force ("Failed to send data on tcp to address '{1}'", Em_error_tcp_send)
			Error_messages.force ("Missing object with ID: '{1}'", Em_error_net_object_missed)

			-- OpenGL
			Error_messages.force ("Unable to initialize GLEW: '{1}'",Em_error_opengl_init_glew)
			Error_messages.force ("Unable to initialize GLSL Shaders: '{1}'",Em_error_opengl_glsl_init)
			Error_messages.force ("Unable to compile a GLSL Shader: '{1}'",Em_error_opengl_glsl_compile)
			Error_messages.force ("Unable to link a GLSL Shader: '{1}'",Em_error_opengl_glsl_link)
			Error_messages.force ("Unable to initialize a framebuffer object: '{1}'",Em_error_opengl_fbo_init)
			Error_messages.force ("Unable to initialize a vertexbuffer object: '{1}'",Em_error_opengl_vbo_init)

			-- Em3D
			Error_messages.force ("Em3d error: '{1}'",Em_error_em3d)
			Error_messages.force ("Em3d texture error: '{1}'",Em_error_em3d_texture)
			Error_messages.force ("Em3d light error: '{1}'",Em_error_em3d_light)
			Error_messages.force ("Em3d identified error: '{1}'",Em_error_em3d_identified)
			Error_messages.force ("Em3d mesh factory error: '{1}'",Em_error_em3d_mesh_factory)
			Error_messages.force ("Em3d material factory error: '{1}'",Em_error_em3d_material_factory)
			Error_messages.force ("Em3d resource factory error: '{1}'",Em_error_em3d_resource_factory)
			Error_messages.force ("Em3d resource error: '{1}'",Em_error_em3d_resource)

		end

feature -- Access

	Error_messages: DS_HASH_TABLE [STRING, INTEGER] is
			-- Table which maps error codes to error messages.
		once
			create Result.make (100)
			make_default_messages
		ensure
			error_messages_not_void: Result /= Void
			default_error_message_exists: Result.has (Em_error_default)
		end

invariant

	default_error_message_exists: Error_messages.has (Em_error_default)

end
