indexing
	description: "A simple frame buffer object used for render to texture, shadow-maps, ..."
	date: "$Date: Jul 8, 2006 9:48:33 PM$"
	revision: "$Revision: 1.0$"
	--TODO: Check if something happens if we pass a deleted texture through
class EMGL_FRAME_BUFFER

inherit
	DISPOSABLE
		export {NONE} all
		redefine dispose, default_create end

	EM_CONSTANTS
		export {NONE} all
		undefine default_create end

	EMGL_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEXT_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEW_FUNCTIONS
		export {NONE} all {ANY} is_supported
		undefine default_create end

	EM_SHARED_ERROR_HANDLER
		export {NONE} all
		undefine default_create end

feature {NONE} -- Initialization

	default_create is
			-- Create a new EMGL_FRAME_BUFFER object.
		require else
			extension_is_available: is_supported( "GL_EXT_framebuffer_object" )
		do
			if not is_supported( "GL_EXT_framebuffer_object" ) then
				Error_handler.raise_error (Error_codes.Em_error_opengl_fbo_init, ["Framebuffer objects not supported!"])
			end
			gl_gen_framebuffers_ext_external( 1, $buffer_id )
		end

	dispose is
			-- Free the framebuffer
		do
			gl_delete_framebuffers_ext_external( 1, $buffer_id )
		end
feature {EMGL_FRAME_BUFFER} -- Status
	buffer_id: INTEGER

	target: INTEGER is
			-- Framebuffer target is always em_gl_framebuffer_ext
		once
			result := em_gl_framebuffer_ext
		end

feature -- Status
	is_framge_buffer:BOOLEAN is
			-- The the current id a framebuffer?
		do
			result := gl_is_framebuffer_ext_external( buffer_id ).to_boolean
		end
	check_status: INTEGER is
			-- Check the framebuffer status
		do
			bind_and_save_old
			result:=gl_check_framebuffer_status_ext_external ( target )
			bind_old
		end
	check_status_nobind: INTEGER is
			-- Check the framebuffer status
		do
			result:=gl_check_framebuffer_status_ext_external ( target )
		end

	frozen current_binding:INTEGER is
			-- Get the current fb binding
		do
			result := emgl_get_integer( em_gl_framebuffer_binding_ext )
		end

	attachment_parameter( attachment, p_name: INTEGER ): INTEGER is
			-- get an attachment parameter
		do
			bind_and_save_old
			gl_get_framebuffer_attachment_parameteriv_ext_external ( target, attachment, p_name, $result )
			bind_old
		end

	object_type( attachment: INTEGER ):INTEGER is
			-- type of image attached to framebuffer attachment point
		do
			result := attachment_parameter( attachment, em_gl_framebuffer_attachment_object_type_ext )
		end
	object_name( attachment: INTEGER ):INTEGER is
			-- name of image attached to framebuffer attachment point
		do
			result := attachment_parameter( attachment, em_gl_framebuffer_attachment_object_name_ext )
		end
	texture_level( attachment: INTEGER ):INTEGER is
			-- mipmap level of texture image attached, if object attached is texture.
		do
			result := attachment_parameter( attachment, em_gl_framebuffer_attachment_texture_level_ext )
		end
	texture_cube_map_face( attachment: INTEGER ):INTEGER is
			-- cubemap face of texture image attached, if object attached is cubemap texture.
		do
			result := attachment_parameter( attachment, em_gl_framebuffer_attachment_texture_cube_map_face_ext )
		end
	texture_3d_zoffset( attachment: INTEGER ):INTEGER is
			-- zoffset of texture image attached, if object attached is 3D texture.
		do
			result := attachment_parameter( attachment, em_gl_framebuffer_attachment_texture_3d_zoffset_ext )
		end

feature{NONE} -- Command
	old_buffer_id: INTEGER
	bind_and_save_old is
			-- Bind the current id and safe the old one
			-- DO NOT USE RECURSIVE!
		do
			old_buffer_id := current_binding
			bind
		end
	bind_old is
			-- Bind the old buffer id
		do
			gl_bind_framebuffer_ext_external( target, old_buffer_id )
		ensure
			id_bound: current_binding = old_buffer_id
		end

feature -- Command
	bind is
			-- Bind the framebuffer
		do
			gl_bind_framebuffer_ext_external( target, buffer_id )
		ensure
			id_bound: current_binding = buffer_id
		end

	unbind is
			-- Unbind the framebuffer
		do
			gl_bind_framebuffer_ext_external ( target, 0 )
		ensure
			id_bound: current_binding = 0
		end

	texture1d_nobind( attachment, tex_target, texture_id, level: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- DOES NOT BIND THE CURRENT BUFFER, YOU MIGHT MANIPULATE ANOTHER BUFFER!
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			gl_framebuffer_texture1dext_external( target, attachment, tex_target, texture_id, level )
		end
	texture1d( attachment, tex_target, texture_id, level: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			bind_and_save_old
			gl_framebuffer_texture1dext_external( target, attachment, tex_target, texture_id, level )
			bind_old
		end

	texture2d_nobind( attachment, tex_target, texture_id, level: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- DOES NOT BIND THE CURRENT BUFFER, YOU MIGHT MANIPULATE ANOTHER BUFFER!
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			gl_framebuffer_texture2dext_external( target, attachment, tex_target, texture_id, level )
		end
	texture2d( attachment, tex_target, texture_id, level: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			bind_and_save_old
			gl_framebuffer_texture2dext_external( target, attachment, tex_target, texture_id, level )
			bind_old
		end

	texture3d_nobind( attachment, tex_target, texture_id, level, z_offset: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- DOES NOT BIND THE CURRENT BUFFER, YOU MIGHT MANIPULATE ANOTHER BUFFER!
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			gl_framebuffer_texture3dext( target, attachment, tex_target, texture_id, level, z_offset )
		end
	texture3d( attachment, tex_target, texture_id, level, z_offset: INTEGER ) is
			-- Attaches a texture to the framebuffer
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			bind_and_save_old
			gl_framebuffer_texture3dext( target, attachment, tex_target, texture_id, level, z_offset )
			bind_old
		end

	renderbuffer_nobind( attachment, renderbuffer_target, render_buffer: INTEGER ) is
			-- Attaches a renderbuffer to the framebuffer
			-- DOES NOT BIND THE CURRENT BUFFER, YOU MIGHT MANIPULATE ANOTHER BUFFER!
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			gl_framebuffer_renderbuffer_ext( target, attachment, renderbuffer_target, render_buffer )
		end
	renderbuffer( attachment, renderbuffer_target, render_buffer: INTEGER ) is
			-- Attaches a renderbuffer to the framebuffer
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			bind_and_save_old
			gl_framebuffer_renderbuffer_ext( target, attachment, renderbuffer_target, render_buffer )
			bind_old
		end

end -- class EMGL_FRAME_BUFFER_OBJECT
