indexing
	description: "Description of class"
	documentation: "Documentation of class"
	date: "$Date: Jul 8, 2006 10:34:50 PM$"
	revision: "$Revision: 1.0$"
	author: "thepk"

class EMGL_RENDER_BUFFER

inherit
	DISPOSABLE
		export {NONE} all
		redefine dispose, default_create end

	EM_CONSTANTS
		export {NONE} all
		undefine default_create end

	EMGL_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEXT_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEW_FUNCTIONS
		export {NONE} all {ANY} is_supported
		undefine default_create end

	EM_SHARED_ERROR_HANDLER
		export {NONE} all
		undefine default_create end

create
	make
feature {NONE} -- Initialization

	default_create is
			-- Create a new EMGL_RENDER_BUFFER object.
		require else
			extension_is_available: is_supported( "GL_EXT_framebuffer_object" )
		do
			if not is_supported( "GL_EXT_framebuffer_object" ) then
				Error_handler.raise_error (Error_codes.Em_error_opengl_fbo_init, ["Framebuffer objects not supported!"])
			end
			gl_gen_renderbuffers_ext( 1, $buffer_id )
		end
	make( internalformat, w, h : INTEGER ) is
			-- Create a render buffer
		do
			default_create
			storage( internalformat, w, h )
		end

	dispose is
			-- Free the renderbuffer
		do
			gl_delete_renderbuffers_ext( 1, $buffer_id )
		end

feature {EMGL_RENDER_BUFFER} -- Status
	buffer_id: INTEGER

	target: INTEGER is
			-- Renderbuffer target is always em_gl_renderbuffer_ext
		once
			result := em_gl_renderbuffer_ext
		end

feature -- Status
	is_render_buffer:BOOLEAN is
			-- The the current id a renderbuffer?
		do
			result := gl_is_renderbuffer_ext( buffer_id ).to_boolean
		end

	frozen current_binding: INTEGER is
			-- Get the current rb binding
		do
			result := emgl_get_integer( em_gl_renderbuffer_binding_ext )
		end


	parameter( pname: INTEGER ): INTEGER is
			-- return a parameter
		do
			bind_and_save_old
			gl_get_renderbuffer_parameteriv_ext( target, pname, $result )
			bind_old
		end

	width: INTEGER is
			-- width of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_width_ext )
		end

	height: INTEGER is
			-- height of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_height_ext )
		end

	internal_format: INTEGER is
			-- internal_format of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_internal_format_ext )
		end

	red_size: INTEGER is
			-- red_size of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_red_size_ext )
		end

	green_size: INTEGER is
			-- green_size of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_green_size_ext )
		end

	blue_size: INTEGER is
			-- blue_size of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_blue_size_ext )
		end

	alpha_size: INTEGER is
			-- alpha_size of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_alpha_size_ext )
		end

	depth_size: INTEGER is
			-- depth_size of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_depth_size_ext )
		end

	stencil_size: INTEGER is
			-- stencilsize of the renderbuffer
		do
			result:=parameter( em_gl_renderbuffer_stencil_size_ext )
		end

feature {NONE} -- Internal Command
	old_buffer_id: INTEGER
	bind_and_save_old is
			-- Bind the current id and safe the old one
			-- DO NOT USE RECURSIVE!
		do
			old_buffer_id := current_binding
			bind
		end
	bind_old is
			-- Bind the old buffer id
		do
			gl_bind_renderbuffer_ext( target, old_buffer_id )
		end

feature -- "Instable" commands
	storage_nobind( internalformat, w, h: INTEGER ) is
			-- Set up the storage (internal memory)
			-- YOU HAVE TO BOND THE TARGET ON YOUR OWN!
		do
			gl_renderbuffer_storage_ext( target, internalformat, w, h )
		end

feature -- Command
	bind is
			-- Bind the framebuffer
		do
			gl_bind_renderbuffer_ext( target, buffer_id )
		end

	unbind is
			-- Unbind the framebuffer
		do
			gl_bind_renderbuffer_ext( target, 0 )
		end

	storage( internalformat, w, h: INTEGER ) is
			-- Set up the storage (internal memory)
		do
			bind_and_save_old
			gl_renderbuffer_storage_ext( target, internalformat, w, h )
			bind_old
		end

	attach_to_framebuffer ( attachment: INTEGER; framebuffer: EMGL_FRAME_BUFFER ) is
			-- Attaches the renderbuffer to a framebuffer
			-- attachment is the point where the texture is atatched to the buffer:
			-- em_gl_color_attachment0_ext ... em_gl_color_attachmentn_ext (where n is from 0 to em_gl_max_color_attachments_ext-1)
			-- em_gl_depth_attachment_ext
	        -- em_gl_stencil_attachment_ext
		do
			framebuffer.renderbuffer( attachment, target, buffer_id )
		end


end -- class EMGL_RENDER_BUFFER
