indexing
	description: "Description of class"
	date: "$Date: 2007-04-23 21:36:13 +0200 (Mon, 23 Apr 2007) $"
	revision: "$Revision: 2578 $"

deferred class EMGL_VERTEX_BUFFER_OBJECT

-- TODO: A display list is considerably faster (2-3 times), find out why!!!

inherit
	DISPOSABLE
		export {NONE} all
		undefine default_create
		redefine dispose, default_create end

	EM_CONSTANTS
		export {NONE} all
			{ANY} em_gl_array_buffer_arb, em_gl_element_array_buffer_arb
		undefine default_create end

	EMGL_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEXT_FUNCTIONS
		export {NONE} all
		undefine default_create end

	GLEW_FUNCTIONS
		export {NONE} all
		undefine default_create end

	EM_SHARED_ERROR_HANDLER
		export {NONE} all
		undefine default_create end

feature {NONE} -- Initialization

	default_create is
			-- Create a new EMGL_VERTEX_BUFFER_OBJECT object.
		require else
			vbo_supported: is_vbo_supported
		do
			if not is_vbo_supported then
				Error_handler.raise_error (Error_codes.Em_error_opengl_vbo_init, ["Vertexbuffer objects not supported!"])
			end
			gl_gen_buffers_arb_external( 1, $buffer_id )
		end

feature {EMGL_FRAME_BUFFER} -- Status
	buffer_id: INTEGER
feature {NONE} -- Status
	currently_bound: INTEGER_REF is
			-- Whis buffer is currently bound?
		once
			create result
		end
--	old_binding: INTEGER
	target: INTEGER is
			-- Valid targets are:
			--   * em_gl_array_buffer_arb: to use with glDrawArrays, ... (stores the vertices)
			--   * em_gl_element_array_buffer_arb: to use with glDrawElements, ... (stores the indices)
		deferred
		ensure
			valid_target: result = em_gl_array_buffer_arb or result = em_gl_element_array_buffer_arb
		end
feature
	is_vbo_supported: BOOLEAN is
			-- Is the VBO extension supported?
		once
			result := is_supported( "GL_ARB_vertex_buffer_object" )
		end

feature {NONE}
	dispose is
			-- Delete the vertex buffer object
		do
			if buffer_id>0 then
				gl_delete_buffers_arb( 1, $buffer_id)
			end
			buffer_id := 0
		end

feature -- Status
	is_buffer: BOOLEAN is
			-- Is the current buffer valid, should always be true
		do
			result := gl_is_buffer_arb( buffer_id )>0
		end

feature -- Command
	bind is
			-- Bind the buffer
		require
			vbo_supported: is_vbo_supported
		do
--			old_binding := currently_bound
			gl_bind_buffer_arb( target, buffer_id )
			currently_bound.set_item( buffer_id )
		end
	unbind is
			-- Bind the buffer
		require
			vbo_supported: is_vbo_supported
		do
			gl_bind_buffer_arb( target, 0 )
			currently_bound.set_item( 0 )
		end
	is_bound: BOOLEAN is
			-- Is this Vertex Buffer currently bound?
		do
			result := currently_bound = buffer_id
		end
	data( a_size: INTEGER; some_data: POINTER; a_usage: INTEGER ) is
			-- Pass the data to the vertex buffer
			-- Possiple usage values are:
			-- em_gl_{dynamic|static|stream}_{copy|read|draw}_arb
			--  * static means the vbo will not change its date (specified once and used many times)
			--  * dynamic means the vbo will change frequently (specified and used repeatedly)
			--  * stream means the data will be changed every frame (specified once and used once)
			--  * draw means the data will be sent to GPU in order to draw (application to GL)
			--  * read means the data will be read by the client's application (GL to application)
			--  * copy means the data will be used both drawing and reading (GL to GL).
		require
			vbo_supported: is_vbo_supported and is_buffer
			vbo_bound: is_bound
		do
			gl_buffer_data_arb (target, a_size, some_data, a_usage )
		end

	sub_data( an_offset,a_size: INTEGER; some_data: POINTER ) is
			-- like data, sub_data is used to copy data into VBO, but it only replaces a range of
			-- data into the existing buffer, starting from the given offset. (The total size of
			-- the buffer must be set by data before using sub_data.)
		require
			vbo_supported: is_vbo_supported and is_buffer
			vbo_bound: is_bound
		do
			gl_buffer_sub_data_arb( target, an_offset, a_size, some_data )
		end

	get_sub_data( an_offset,a_size: INTEGER; some_data: POINTER ) is
			-- The inverse function to sub_data. It stores some data from the vertex buffer into
			-- the memory at position some_data
		require
			vbo_supported: is_vbo_supported and is_buffer
			vbo_bound: is_bound
		do
			gl_get_buffer_sub_data_arb ( target, an_offset, a_size, some_data )
		end

	map( an_access: INTEGER ): POINTER is
			-- This is the heart of the vertex buffer, by using map you get access to the vertex buffers
			-- memory directly
			-- access can be em_gl_read_only_arb, em_gl_write_only_arb or em_gl_read_write_arb
		require
			vbo_supported: is_vbo_supported and is_buffer
			vbo_bound: is_bound
		do
			result := gl_map_buffer_arb (target, an_access)
			if result.to_integer_32 = em_gl_false then
				Error_handler.raise_error (Error_codes.Em_error_opengl_vbo_init, ["Map failed!"])
			end
		end
	unmap is
			-- Don't forget to unmap the target when you don't need it anymore
		require
			vbo_supported: is_vbo_supported and is_buffer
			vbo_bound: is_bound
		do
			if gl_unmap_buffer_arb (target)=em_gl_false then
				Error_handler.raise_error (Error_codes.Em_error_opengl_vbo_init, ["Unmaping failed"])
			end
		end

	get_parameter( pname: INTEGER ): INTEGER is
			-- Get a VBO parameter
			-- possible parameters are:
			-- em_gl_buffer_size_arb, em_gl_buffer_usage_arb,
			-- em_gl_buffer_access_arb, or em_gl_buffer_mapped_arb
		require
			vbo_supported: is_vbo_supported and is_buffer
		do
			bind
			gl_get_buffer_parameteriv_arb (target, pname, $result)
			unbind
		end

	get_pointer( pname: INTEGER ): POINTER is
			-- Get a VBO pointer parameter
			-- the only supported value is em_gl_buffer_map_pointer_arb
		require
			vbo_supported: is_vbo_supported and is_buffer
		do
			bind
			gl_get_buffer_pointerv_arb (target, pname, $result)
			unbind
		end
feature --Attributes
	size: INTEGER is
			-- The size of the buffer
		do
			result := get_parameter( em_gl_buffer_size )
		end
	usage: INTEGER is
			-- The usage of the buffer
		do
			result := get_parameter( em_gl_buffer_usage )
		end
	access: INTEGER is
			-- The access mode of the buffer
		do
			result := get_parameter( em_gl_buffer_access )
		end
	mapped: BOOLEAN is
			-- The access mode of the buffer
		do
			result := get_parameter( em_gl_buffer_mapped ) = em_gl_true
		end
	map_pointer: POINTER is
			-- Get the map Pointer
		do
			result := get_pointer( em_gl_buffer_map_pointer )
		end


invariant
	extension_is_available: is_buffer implies is_vbo_supported

	-- The problem with this invariant is:
	--  "A name returned by glGenBuffers, but not yet associated with a buffer object
    --   by calling glBindBuffer, is not the name of a buffer object."
--	buffer_is_valid: buffer_id /= 0 implies is_buffer
end -- class EMGL_VERTEX_BUFFER_OBJECT
