indexing
	description: "[
	
		Decorator boxes that have a flat or 3-dimensional, colored background.
		Contents can be aligned. This only makes sense if the size is set to
		something greater than the size of the contents. The size of this box
		will always be adjusted to completely accommodate the contents.
		Use put_depth/prune_depth to change the appearance of the background from
		flat to 3d and vice versa.
		Do not use Align_user_specified when specifing the alignment, as it might
		give unexpected results.

	]"
	author: ""
	date: "$Date: 2007-02-12 05:27:06 +0100 (Mon, 12 Feb 2007) $"
	revision: "$Revision: 2503 $"

class
	EM_VIZ_BACKGROUND_BOX [B -> EM_VIZ_BOX]

inherit
	EM_VIZ_DECORATOR_BOX [B]
		redefine
			vz_render, resize_container, realign_contents
		end

	EM_VIZ_COLORED

	EM_VIZ_ALIGNABLE

	EM_VIZ_RESIZEABLE

create

	make

feature -- Initialization

	make (a_size: like size) is
			-- Make new background box with minimal `a_size' and default background
			-- color and alignment.
		do
			make_decorator
			make_colored (Viz_options.default_background_color)
			make_alignable
			set_alignment ([Align_center, Align_center, Align_center])
			make_resizeable (a_size)
			prune_depth
		end

feature -- Status report

	has_depth: BOOLEAN
			-- 3d-background?

feature -- Status setting

	put_depth is
			-- Enabled 3d-background
		do
			has_depth := True
			expire
		ensure
			has: has_depth
			expired: needs_update
		end

	prune_depth is
			-- Disable 3d-background
		do
			has_depth := False
			expire
		ensure
			pruned: not has_depth
			expired: needs_update
		end

feature {NONE} -- Implementation

	vz_render (viewing_direction: EM_VECTOR3D) is
			-- Render background and child item, if any
		local
			e00, e10, e11, e01: like size
		do
			vz_push_attributes (Vz_polygon_bit | Vz_depth_buffer_bit)
			vz_set_depth_test (False, False)

			if has_depth then
				vz_set_culling (True, Vz_front)
				render_colored_cube (bounds.min, bounds.max, color)
			else
				vz_set_culling (False, Vz_front_and_back)
				e00 := bounds.min
				e11 := bounds.max
				e11.set_z (e00.z)
				e10 := e00
				e10.set_x (e11.x)
				e01 := e00
				e01.set_y (e11.y)

				vz_begin_quads
					vz_color (color)
					vz_normal3d (0, 0, -1)
					vz_vertex (e00)
					vz_vertex (e10)
					vz_vertex (e11)
					vz_vertex (e01)
				vz_end
			end

			vz_pop_attributes
			Precursor (viewing_direction)
		end

	resize_container is
			-- Resize current to fit contents
		local
			s: like size
		do
			s := size
			s.ew_maximum (content_size)
			set_size_internal (s)
		end

	realign_contents is
			-- Realign contents inside current
		local
			b: like bounds
		do
			b.set ([0.0,0.0,0.0], bounds.size)
			if item /= Void then
				item.realign (b, alignment)
			end
		end

end
