	indexing
	description: "[

		Boxes are the building blocks of Viz. The basic box has a name and a size.
		It may also have a parent if it is embedded in a collection box.
		Although each box features a axis-aligned bounding-box, only its size is
		effectively exposed. Position only is meaningful if a box is contained in
		a collection box.

	]"
	author: ""
	date: "$Date: 2007-02-12 06:18:18 +0100 (Mon, 12 Feb 2007) $"
	revision: "$Revision: 2504 $"

deferred class
	EM_VIZ_BOX

inherit

	EM_VIZ_RENDERABLE
		redefine
			vz_setup, expire
		end

	EM_VIZ_NAMED

	EM_VIZ_SIZED

feature -- Initialization

	make_box is
			-- Initialize
		do
			make_renderable
			make_anonymous
			parent := Void
		ensure
			orphan_at_birth: parent = Void
		end

feature -- Access

	parent: EM_VIZ_COLLECTION_BOX [EM_VIZ_BOX]
			-- Parent box that contains current

	width: DOUBLE is
			-- Width of box
		do
			Result := bounds.width
		end

	height: DOUBLE is
			-- Height of box
		do
			Result := bounds.height
		end

	depth: DOUBLE is
			-- Depth of box
		do
			Result := bounds.depth
		end

	size: EM_VECTOR3D is
			-- Size of box
		do
			Result := bounds.size
		end

feature -- Update mechanism

	expire is
			-- Expire all the way up the hierarchy
		do
			-- Only expire if needed
			if not needs_update then
				Precursor
				-- Tell parent, too
				if parent /= Void then
					parent.expire
				end
			end
		end

feature {EM_VIZ_BOX} -- Rendering

	depth_position: EM_VECTOR3D is
			--
		do
			Result := bounds.center
		end

	setup_local_coordinate_frame is
			-- Setup local coordinate frame such that origin is at bounds.min
		local
			org: EM_VECTOR3D
		do
			org := bounds.min

			vz_push_frame
			vz_translated (org.x, org.y, org.z)
		end

	revert_local_coordinate_frame is
			-- Revert to previous coordinate frame
		do
			vz_pop_frame
		end

feature {NONE} -- Implementation

	vz_setup is
			-- Render bounding box when debugging
		do
			if Viz_options.debug_gfx then
				render_bounding_box (bounds.min, bounds.max)
			end
			Precursor
		end

	set_width_internal (a_width: like width) is
			-- Internal resize handler
		do
			bounds.x.resize (a_width)
		end

	set_height_internal (a_height: like height) is
			-- Internal resize handler
		do
			bounds.y.resize (a_height)
		end

	set_depth_internal (a_depth: like depth) is
			-- Internal resize handler
		do
			bounds.z.resize (a_depth)
		end

	set_size_internal (a_size: like size) is
			-- Internal resize handler
		do
			bounds.resize (a_size)
		end

feature {EM_VIZ_COLLECTION_BOX} -- Implementation

	set_parent (a_parent: like parent) is
			-- Set parent (accessible to collection box only!)
		do
			parent := a_parent
			if needs_update and parent /= Void then
				parent.expire
			end
		ensure
			set: parent = a_parent
		end

	realign (parent_bounds: EM_INTERVAL_3D; alignment: EM_ALIGNMENT_3D) is
			-- Realign inside `parent_bounds' according to `alignment'
		do
			bounds.align_with (parent_bounds, alignment)
		end

	reposition (a_position: like size) is
			-- Reposition current
		do
			bounds.translate (a_position - bounds.min)
		end

feature {EM_VIZ_BOX} -- Implementation

	bounds: EM_INTERVAL_3D
			-- Bounding box

invariant

	parent_relationship: parent /= Void implies parent.has (Current)

end
