indexing
	description: "[

		A box that is used to decorate a single child box.
		This box always has one child, even if the child should be Void.

	]"
	author: ""
	date: "$Date: 2007-02-19 11:34:20 +0100 (Mon, 19 Feb 2007) $"
	revision: "$Revision: 2545 $"

deferred class
	EM_VIZ_DECORATOR_BOX [B -> EM_VIZ_BOX]

inherit
	EM_VIZ_COLLECTION_BOX [B]

feature -- Initialization

	make_decorator is
			-- Initialize
		do
			make_collection
			create single_item.make	(1)
			single_item.put_last (Void)
		end

feature -- Access

	as_linear: DS_LINEAR [B] is
			-- Linear access
		do
			Result := single_item
		end

	item: B assign put is
			-- Child item
		do
			Result := single_item.item (1)
		end

feature -- Measurement

	count: INTEGER is
			-- Always has child, even if item = Void
		do
			Result := 1
		end

feature -- Status report

	is_empty: BOOLEAN is
			-- Never empty because count = 1
		do
			Result := count = 0
		end

	has (b: B): BOOLEAN is
			-- Has item `b'?
		do
			Result := b = item
		end

feature -- Element change

	put (b: like item) is
			-- Set child box `b'
		do
			cast_out (item)
			single_item.replace (b, 1)
			adopt (b)
		ensure
			has: has (b)
		end

feature -- Removal

	prune is
			-- Remove child and set Void as child box
		do
			put (Void)
		end

feature -- Measurements

	content_size: like size is
			-- Measurements of child box
		do
			if item /= Void then
				Result := item.size
			else
				Result := Result.zero
			end
		end

feature {NONE} -- Implementation

	vz_render (viewing_direction: EM_VECTOR3D) is
			-- Render child
		do
			setup_local_coordinate_frame
			if item /= Void then
				item.render (viewing_direction)
			end
			revert_local_coordinate_frame
		end

	single_item: DS_ARRAYED_LIST [B]
			-- Storage

	resize_container is
			-- Resize container for update
		do
			set_size_internal (content_size)
		end

	realign_contents is
			-- Realign child after resize
		do
			if item /= Void then
				reposition ([0.0, 0.0, 0.0])
			end
		end

end
