indexing
	description: "[
	
		A box whose contents can be lit by diffuse lighting.
		The default light direction points away from the viewer, which
		means that polygons facing the viewer will be fully lit.

	]"
	author: ""
	date: "$Date: 2007-02-18 01:09:05 +0100 (Sun, 18 Feb 2007) $"
	revision: "$Revision: 2532 $"

class
	EM_VIZ_LIGHT_BOX [B -> EM_VIZ_BOX]

inherit
	EM_VIZ_DECORATOR_BOX [B]
		redefine
			vz_setup, vz_revert
		end

create
	make

feature -- Initialization

	make is
			-- Initialize `Current'.
		do
			make_decorator
			set_lighting_enabled (True)
			set_light_color (Viz_options.default_light_color)
			set_light_direction ([0.0, 0.0, 1.0])
		end

feature -- Access

	lighting_enabled: BOOLEAN
			-- Should lighting be enabled?

	light_direction: EM_VECTOR3D
			-- Direction of light-rays

	light_color: EM_COLOR
			-- Color of light

feature -- Element change

	set_lighting_enabled (enable: like lighting_enabled) is
			-- Set lighting enabled/disabled
		do
			lighting_enabled := enable
			expire
		ensure
			set: lighting_enabled = enable
			expired: needs_update
		end

	set_light_direction (d: like light_direction) is
			-- Set light parameter
		do
			light_direction := d
			expire
		ensure
			-- set: light_direction = d
			expired: needs_update
		end

	set_light_color (c: like light_color) is
			-- Set light parameter
		do
			light_color := c
			expire
		ensure
			-- set: light_color = c
			expired: needs_update
		end

feature {NONE} -- Implementation

	vz_setup is
			-- Setup vz-state (opengl)
		do
			vz_push_attributes (Vz_lighting_bit)
			vz_set_lighting (True)
			vz_set_light (light_color, light_direction)
			Precursor
		end

	vz_revert is
			-- Revert vz-state (opengl)
		do
			Precursor
			vz_pop_attributes
		end

end
