indexing
	description: "[

		A box that sets up the neccessary rendering environment to
		plot a sequence of n-dimensional values with a user-specified
		plotter. This is the base class for all plots.
		Nothing will be rendered if no plotter exists.

	]"
	author: ""
	date: "$Date: 2007-02-18 01:09:05 +0100 (Sun, 18 Feb 2007) $"
	revision: "$Revision: 2532 $"

deferred class
	EM_VIZ_PLOT [D -> EM_ABSTRACT_VECTOR [DOUBLE], R -> EM_VIZ_PLOTTER [D]]

inherit
	EM_VIZ_RENDERER_BOX
		redefine
			update
		end

feature -- Initializaion

	make_plot (a_size: like size; an_interval: like interval) is
			-- Initialize plot
		do
			make_renderer_box (a_size, an_interval)
			create_data
		end

feature -- Access

	plotter: R
			-- Plotter that plots data

feature {R} -- Protected Accesss

	data: DS_LINEAR [D]
			-- Data to be plotted

feature -- Element change

	set_plotter (a_plotter: like plotter) is
			-- Change plotter to `a_plotter'
		do
			plotter := a_plotter
			expire
		ensure
			set: plotter = a_plotter
			expired: needs_update
		end

feature -- Update mechansim

	update is
			-- Update data if neccessary
		do
			Precursor
			update_data
		ensure then
			data_exists: data /= Void
		end

feature {EM_VIZ_PLOT} -- Update implementation

	create_data is
			-- Create data
		deferred
		ensure
			data_exists: data /= Void
		end

	update_data is
			-- Update data
		deferred
		end

feature -- Rendering

	vz_render (viewing_direction: EM_VECTOR3D) is
			-- Render if plotter exists
		do
			if plotter /= Void then
				plotter.plot (data)
			end
		end

feature {NONE} -- Implementation

invariant

	data_exists: data /= Void

end
