indexing

	description: "[

		Global options for all widgets.
		
		Use EM_SHARED_WIDGET_OPTIONS to access the singleton instance.

	]"
	date: "$Date: 2005-10-23 13:21:02 +0200 (Sun, 23 Oct 2005) $"
	revision: "$Revision: 1137 $"

class EM_WIDGET_OPTIONS

create {EM_SHARED_WIDGET_OPTIONS}

	make

feature {NONE} -- Initialisation

	make is
			-- Initialise default values.
		do
			are_tooltips_enabled := True
			tooltip_show_delay := 1000
			is_transparency_refresh_enabled := False
			is_dialog_dragging_restriction_enabled := True
		end

feature -- Access

	tooltip_show_delay: INTEGER
			-- Time to wait before tooltip is shown

feature -- Status report

	are_tooltips_enabled: BOOLEAN
			-- Are tooltips enabled?

	is_transparency_refresh_enabled: BOOLEAN
			-- Are transparent parts always refresh?
			-- When transparency is used but a constant refresh of transparent areas is not
			-- necessary, disable this option to improve performance.

	is_dialog_dragging_restriction_enabled: BOOLEAN
			-- Are dialogs restricted to coordinates inside scene?

feature -- Status setting

	enable_tooltips is
			-- Enable tooltips.
		do
			are_tooltips_enabled := True
		ensure
			tooltips_enabled: are_tooltips_enabled
		end

	disable_tooltips is
			-- Disable tooltips
		do
			are_tooltips_enabled := False
		ensure
			tooltips_disabled: not are_tooltips_enabled
		end

	enable_transparency_refresh is
			-- Enable transparency refresh.
		do
			is_transparency_refresh_enabled := True
		ensure
			transparency_refresh_enabled: is_transparency_refresh_enabled
		end

	disable_transparency_refresh is
			-- Disable transparency refresh.
		do
			is_transparency_refresh_enabled := False
		ensure
			transparency_refresh_disabled: not is_transparency_refresh_enabled
		end

	enable_dialog_dragging_restriction is
			-- Enable restriction of dialog dragging.
		do
			is_dialog_dragging_restriction_enabled := True
		ensure
			dialog_dragging_restriction_enabled: is_dialog_dragging_restriction_enabled
		end

	disable_dialog_dragging_restriction is
			-- Disable restriction of dialog dragging.
		do
			is_dialog_dragging_restriction_enabled := False
		ensure
			dialog_dragging_restriction_disabled: not is_dialog_dragging_restriction_enabled
		end

feature -- Element change

	set_tooltip_show_delay (a_value: like tooltip_show_delay) is
			-- Set `tooltip_show_delay' to `a_value'.
		do
			tooltip_show_delay := a_value
		ensure
			tooltip_show_delay_set: tooltip_show_delay = a_value
		end

end
