/// Author: Weiser Robert
/// Version: 0.0.3
/// Last Modified: 02.08.2005, 12:20
/// Descrition:	This is a script which will process installer template files
///		and create the final installer files



///------------ File Names ------------///
/// Info Files
version_info = "version";
installer_info = "installer_info.ini";
installer_info_temp = "installer_info_temp.ini";

/// Student Installer
stud_installer_info = "installer_stud_info.iss";
stud_installer_code = "installer_stud_code.iss";
stud_installer = "installer_stud.iss";

/// Admin Application Installer
admin_app_installer_info = "installer_admin_app_info.iss";
admin_app_installer_code = "installer_admin_app_code.iss";
admin_app_installer = "installer_admin_app.iss";

/// Admin Library Installer
admin_lib_installer_info = "installer_admin_lib_info.iss";
admin_lib_installer_code = "installer_admin_lib_code.iss";
admin_lib_installer = "installer_admin_lib.iss";


///------------ File Operation ------------///
ForReading = 1
ForWriting = 2
ForAppending = 8

///------------ Modifiers ------------///
mod = "MOD: ";
ver = "version"; 


///------------ Entry Point ------------///
main();


function main()
{
	Stage1();
	Stage2();	
	CleanUp();
}

function Stage1()
///replace the version-tag in the installer_info-file by the real version
{
	ReplaceVersionInfo();
}

function Stage2()
///process all template installer files and create the final installer
///files
{
	ProcessStudentInstaller();
	ProcessAdminLibInstaller();
	ProcessAdminAppInstaller();
}





///------------ Stage 1 ------------///

function ReplaceVersionInfo()
///substitue all 'ver'-strings in the ini file by the real version number
{
	var fso;
	
	///create a new temporary file
	MoveFile(installer_info, installer_info_temp);

	fso = WScript.CreateObject("Scripting.FileSystemObject");
	source = fso.OpenTextFile(installer_info_temp, ForReading, false);
	dest = fso.CreateTextFile(installer_info, true);
	
	///replace the original file by injecting the correct version info
	while (source.AtEndOfStream != true)
	{
		var line = source.Readline();
		
		if (NeedsToBeSubstituted(line))
		{
			line = GetSubstitutedLine(line);
		}
		
		dest.WriteLine(line);
	}
	
	source.Close();
	dest.Close();	
}

function NeedsToBeSubstituted(line)
///check wheter the line contains a version-modifier
{
	return LineHasSubstring(line, ver);
}

function GetSubstitutedLine(line)
///return the line with the substitued version-string
{
	var version;
	
	version = GetVersion();
	
	return(line.replace(ver, version));
}

function GetVersion()
///return the version
{
	var fso, f, version;
	
	fso = WScript.CreateObject("Scripting.FileSystemObject");
	f = fso.OpenTextFile(version_info);
	version = f.ReadLine();
	f.Close();
	
	return(version);
}





///------------ Stage 2 ------------///

function ProcessStudentInstaller()
///process the student installer templates
{
	ProcessTemplateFile(stud_installer_info, stud_installer);
	AppendFile(stud_installer_code, stud_installer);
}

function ProcessAdminLibInstaller()
///process the admin lib installer templates
{
	ProcessTemplateFile(admin_lib_installer_info, admin_lib_installer);
	AppendFile(admin_lib_installer_code, admin_lib_installer);
}

function ProcessAdminAppInstaller()
///process the admin app installer templates
{
	ProcessTemplateFile(admin_app_installer_info, admin_app_installer);
	AppendFile(admin_app_installer_code, admin_app_installer);
}

function CleanUp()
///remove all unneccesary files
{
	var fso, f;
	
	fso = WScript.CreateObject("Scripting.FileSystemObject");
	
	f = fso.GetFile(stud_installer_info);
	f.Delete();
	
	f = fso.GetFile(stud_installer_code);
	f.Delete();
	
	f = fso.GetFile(admin_app_installer_info);
	f.Delete();
	
	f = fso.GetFile(admin_app_installer_code);
	f.Delete();
	
	f = fso.GetFile(admin_lib_installer_info);
	f.Delete();
	
	f = fso.GetFile(admin_lib_installer_code);
	f.Delete();
	
	f = fso.GetFile(installer_info_temp);
	f.Delete();

	f = fso.GetFile(installer_info);
	f.Delete();

	f = fso.GetFile("installer_generator.js");
	f.Delete();
}

function ProcessTemplateFile(source_name, dest_name)
///process the template file by substituing all needed information
///and write it to a new file
{
	var fso, source, dest;
	
	fso = WScript.CreateObject("Scripting.FileSystemObject");
	source = fso.OpenTextFile(source_name, ForReading, false);
	dest = fso.CreateTextFile(dest_name, true);
	
	while (source.AtEndOfStream != true)
	{
		var line = source.Readline();
		
		if (NeedsToBeModified(line))
		{
			line = GetModifiedLine(line);
		}
		
		dest.WriteLine(line);
	}
	
	source.Close();
	dest.Close();
}

function NeedsToBeModified(line)
///check if the given line need to be modified
{
	return LineHasSubstring(line, mod);
}

function LineHasSubstring(line, substr)
///check if given line has substring 'substr'
{
	var reg = new RegExp(substr,"i");
	
	if (line.match(reg) != null)
		return (true);
	else
		return (false);
}

function GetModifiedLine(line)
///return the modified line
{
	var param = GetParam(mod, line);
	var value = GetValue(param)
	
	return(value);
}

function GetParam(tag, line)
///return the parameter which needs to be modifed
{
	return (line.substring(tag.length,line.length));
}

function GetValue(param)
///return value assigned to the given parameter
{
	var fso, f;

	fso = WScript.CreateObject("Scripting.FileSystemObject");
	f = fso.OpenTextFile(installer_info, ForReading, false);
	
	while (f.AtEndOfStream != true)
	{
		var line = f.Readline();
		
		if (LineHasSubstring(line, param))
			return(line.substring((param.length+2), line.length));
	}
	
	return ("");
}





///------------ Helper Functions ------------///

function MoveFile(source, dest)
///move file from source to dest
{
	var fso, f;

	fso = WScript.CreateObject("Scripting.FileSystemObject");
	f = fso.GetFile(source);
	f.Move(dest);
}

function AppendFile(source_name, dest_name)
//append source-file to dest-file
{
	var fso, source, dest;
	
	fso = WScript.CreateObject("Scripting.FileSystemObject");
	source = fso.OpenTextFile(source_name, ForReading);
	dest = fso.OpenTextFile(dest_name, ForAppending);
	
	while (source.AtEndOfStream != true)
	{
		var line = source.Readline();
		dest.WriteLine(line);
	}
	
	source.Close();
	dest.Close();
}
