indexing

	description: "[

		XML filter which stores the elements of an XML-file in a hash table (sections)
		of a linked list (the entries on a section) of hash tables (an xml entry with
		all it's atributes).
		Supports tag "Defaults" to set prefixes for the xml-attributes.
		
		Structure of XML-File must be:
		---------------------------------------------------------------------------
		<?xml version="1.0"?>
		
		<abc>
			(<Section name="cde">
				(<Prefix fgh="fgh" ijk="ijk"/>)*
					(<xyz fgh="FGH" ijk="IJK" lmn="LMN"/>)*
			</Section>)+
		</abc>
		---------------------------------------------------------------------------
		
		This will generate a entry in the hashtable for every section (here "cde"), and an entry in
		the linked list for every "xyz".
		The hashtable in the linked list will contain all attributes (here "fgh"->"fghFGH", "ijk"->"ijkIJK" and "lmn"->"LMN")

	]"
	date: "$Date: 2005-10-26 12:51:11 +0200 (Wed, 26 Oct 2005) $"
	revision: "$Revision: 1286 $"

class EM_CLASS_GENERATOR_XM_FILTER

inherit

	XM_CALLBACKS_FILTER
	redefine
		on_start,
		on_error,
		on_start_tag,
		on_attribute,
		on_start_tag_finish,
		on_end_tag
	end

create

	make_null

feature -- Document

	on_start is
			-- Called when parsing starts.
		do
			create xml_table.make_default

			create prefixes.make_default
			prefixes.set_equality_tester (create {KL_EQUALITY_TESTER[STRING]})
		end

feature -- Errors

	on_error (a_message: STRING) is
			-- Event producer detected an error.
		do
			has_error := true
			io.put_string ("Parse error:%N" + a_message + "%N")
		end

feature -- Tag

	on_start_tag (a_namespace: STRING; a_prefix: STRING; a_local_part: STRING) is
			-- Start of start tag.
		do
			current_tag := a_local_part
			if current_section /= void then
				if xml_table.has (current_section) and not current_tag.as_lower.is_equal ("prefix") then
					xml_table.item (current_section).put_first (create {DS_HASH_TABLE[STRING, STRING]}.make_default)
					xml_table.item (current_section).first.put(current_tag, "tag")
				elseif current_tag.as_lower.is_equal ("prefix") then
					prefixes.wipe_out
				end
			end
		end

	on_attribute (a_namespace: STRING; a_prefix: STRING; a_local_part: STRING; a_value: STRING) is
			-- Start of attribute.
		local
			full_value: STRING
		do
			if current_tag.as_lower.is_equal ("section") and then a_local_part.as_lower.is_equal ("name") then
				if a_value.as_lower.is_equal ("all") then
					on_error ("%"all%" is a keyword")
				end
				current_section := a_value
				-- Create a new section
				xml_table.force (create {DS_LINKED_LIST[DS_HASH_TABLE[STRING, STRING]]}.make, current_section)
			elseif current_tag.as_lower.is_equal ("prefix") and not prefixes.has (a_local_part.as_lower) then
				-- Add new prefix
				prefixes.force (a_value, a_local_part.as_lower)
			else
				-- Add attribute of XML entry to current
				if current_section /= void then
					if prefixes.has (a_local_part) then
						-- prepend prefix if it exists
						full_value := prefixes.item (a_local_part) + a_value
					else
						full_value := a_value
					end
					xml_table.item(current_section).first.force(full_value, a_local_part)
				else
					on_error ("Section not specified")
				end
			end
		end

	on_start_tag_finish is
			-- End of start tag.
		do
			current_tag := void
		end

	on_end_tag (a_namespace: STRING; a_prefix: STRING; a_local_part: STRING) is
			-- End tag.
		do
			if a_local_part.as_lower.is_equal ("section") then
				current_section := Void
			end
		end

	has_error: BOOLEAN

	prefixes: DS_HASH_TABLE[STRING, STRING]
		-- Current prefixes

	current_tag: STRING
	current_section: STRING

	xml_table: DS_HASH_TABLE[DS_LINKED_LIST[DS_HASH_TABLE[STRING, STRING]], STRING]
		-- Data structure to store the xml entries
end
